package fr.janua.client.presenter;

import java.util.List;
import java.util.Set;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.Window;
import com.google.web.bindery.event.shared.SimpleEventBus;
import com.google.web.bindery.requestfactory.shared.Receiver;
import com.google.web.bindery.requestfactory.shared.Request;
import com.mvp4g.client.annotation.Presenter;
import com.mvp4g.client.presenter.LazyPresenter;

import fr.janua.client.CrudRequestFactory;
import fr.janua.client.JaguardsEventBus;
import fr.janua.client.UserManager;
import fr.janua.client.internationalization.JaguardsConstants;
import fr.janua.client.proxy.GroupProxy;
import fr.janua.client.proxy.SiteProxy;
import fr.janua.client.proxy.TypeProxy;
import fr.janua.client.stub.GroupRequest;
import fr.janua.client.stub.SiteRequest;
import fr.janua.client.stub.TypeRequest;
import fr.janua.client.view.PageGroupView;
import fr.janua.client.view.interfaces.IPageGroupView;
import fr.janua.client.view.interfaces.IPageGroupView.IPageGroupPresenter;
import fr.janua.shared.Action;

@Presenter(view = PageGroupView.class)
public class PageGroupPresenter extends LazyPresenter<IPageGroupView, JaguardsEventBus> implements IPageGroupPresenter {

	private JaguardsConstants constants = GWT.create(JaguardsConstants.class);
	private CrudRequestFactory requestFactory = GWT.create(CrudRequestFactory.class);

	public void onGoToPageGroup() {
		eventBus.setBody(view);
		view.loadPage();
		getAllSites();
		getAllTypes();
	}

	
	/* CRUD */
	
	


	@Override
	public void getAllGroup() {
		if(requestFactory.getEventBus() == null){
			requestFactory.initialize(new SimpleEventBus());
		}

		// 1 - On récupère tous les Sites
		SiteRequest siteRequest = requestFactory.getSiteRequest();
		Request<List<SiteProxy>> request1 = siteRequest.findAllSite();
		request1.fire(new Receiver<List<SiteProxy>>() {
			@Override
			public void onSuccess(final List<SiteProxy> listSite) {
				
				// 2 - On récupère tous les Types
				TypeRequest typeRequest = requestFactory.getTypeRequest();
				Request<List<TypeProxy>> request2 = typeRequest.findAllType();
				request2.fire(new Receiver<List<TypeProxy>>() {
					@Override
					public void onSuccess(final List<TypeProxy> listType) {
						
						// 3 - On récupère tous les Groupes
						GroupRequest groupRequest = requestFactory.getGroupRequest();
						Request<List<GroupProxy>> request = groupRequest.findAllGroup().with("siteAllowed", "typeAllowed");
						request.fire(new Receiver<List<GroupProxy>>() {
							@Override
							public void onSuccess(List<GroupProxy> listGroup) {
								eventBus.addLog(Action.READ_GROUP, "[all]");
								// 4 - Maintenant qu'on a les Groupes, les Sites et les Types, on envoie tout à la vue
								view.displayGroups(listSite, listType, listGroup);
							}
						});
					}
				});
			}
		});
	}
	
	@Override
	public void createGroup(final String name, String mail, Set<SiteProxy> setSite, Set<TypeProxy> setType, int numCrudUser, int numCrudMessage, int numCrudGroup, int numCrudSite, int numCrudType, int numCrudLog, int numCrudUploadDoc) {
		if(requestFactory.getEventBus() == null){
			requestFactory.initialize(new SimpleEventBus());
		}
		
		GroupRequest groupRequest = requestFactory.getGroupRequest();
		
		GroupProxy groupProxy = groupRequest.create(GroupProxy.class);
		groupProxy.setName(name);
		groupProxy.setMail(mail);
		groupProxy.setSiteAllowed(setSite);
		groupProxy.setTypeAllowed(setType);
		groupProxy.setCrudUser(numCrudUser);
		groupProxy.setCrudMessage(numCrudMessage);
		groupProxy.setCrudGroup(numCrudGroup);
		groupProxy.setCrudSite(numCrudSite);
		groupProxy.setCrudType(numCrudType);
		groupProxy.setCrudLog(numCrudLog);
		groupProxy.setCrudUploadDoc(numCrudUploadDoc);
		
		Request<Boolean> request = groupRequest.create(groupProxy);
		request.fire(new Receiver<Boolean>() {
			@Override
			public void onSuccess(Boolean response) {
				if(response){
					eventBus.addLog(Action.CREATE_GROUP, "succès [nom:" + name + "]");
					Window.alert(constants.successCreateGroup());
					view.loadPage();
				}else{
					eventBus.addLog(Action.CREATE_GROUP, "échec [nom:" + name + "]");
					Window.alert(constants.failCreateGroup());
				}
			}
		});
	}
	

	@Override
	public void updateGroup(final String name, final String mail, final Set<SiteProxy> setSite, final Set<TypeProxy> setType, final int numCrudUser, final int numCrudMessage, final int numCrudGroup, final int numCrudSite, final int numCrudType, final int numCrudLog, final int numCrudUploadDoc) {
		if(requestFactory.getEventBus() == null){
			requestFactory.initialize(new SimpleEventBus());
			System.out.println("INIT EventBus");
		}
		final GroupRequest groupRequest1 = requestFactory.getGroupRequest();

		// 1 - on récupère le GroupProxy
		final Request<GroupProxy> request = groupRequest1.findByName(name);

		request.fire(new Receiver<GroupProxy>() {
			@Override
			public void onSuccess(GroupProxy response) {
				// 2 - on enlève le frozen de response, et on crée un 2nd context appelé groupRequest2(groupRequest1 étant occupé)
				GroupRequest groupRequest2 = requestFactory.getGroupRequest();
				GroupProxy groupUpdated = groupRequest2.edit(response);

				// 3 - on modifie le GroupProxy récupéré dans la BDD
				groupUpdated.setMail(mail);
				groupUpdated.setSiteAllowed(setSite);
				groupUpdated.setTypeAllowed(setType);
				groupUpdated.setCrudUser(numCrudUser);
				groupUpdated.setCrudMessage(numCrudMessage);
				groupUpdated.setCrudGroup(numCrudGroup);
				groupUpdated.setCrudSite(numCrudSite);
				groupUpdated.setCrudType(numCrudType);
				groupUpdated.setCrudLog(numCrudLog);
				groupUpdated.setCrudUploadDoc(numCrudUploadDoc);
				
				// 4 - on le renvoie avec les nouvelle valeurs
				Request<Boolean> request2 = groupRequest2.update(groupUpdated);

				request2.fire(new Receiver<Boolean>() {
					@Override
					public void onSuccess(Boolean response) {
						if(response){
							// so on actualise notre propre groupe : on refresh notre utilisateur courant
							if(name.equals(UserManager.getInstance().getGroupName()))
								eventBus.refreshUserWithoutChangePassword();
							eventBus.addLog(Action.UPDATE_GROUP, "succès [nom:" + name + "]");
							Window.alert(constants.successUpdateGroup());
							view.loadPage();
						}else{
							eventBus.addLog(Action.UPDATE_GROUP, "échec [nom:" + name + "]");
							Window.alert(constants.failUpdateGroup());
						}
					}
				});
			}
		});
		
	}
	
	
	
	
	private void getAllTypes() {
		if(requestFactory.getEventBus() == null){
			requestFactory.initialize(new SimpleEventBus());
		}
		
		TypeRequest typeRequest = requestFactory.getTypeRequest();
		Request<List<TypeProxy>> request = typeRequest.findAllType();
		
		request.fire(new Receiver<List<TypeProxy>>() {
			@Override
			public void onSuccess(List<TypeProxy> listTypes) {
				view.setListTypes(listTypes);
			}
		});
	}


	private void getAllSites() {
		if(requestFactory.getEventBus() == null){
			requestFactory.initialize(new SimpleEventBus());
		}
		
		SiteRequest siteRequest = requestFactory.getSiteRequest();
		Request<List<SiteProxy>> request = siteRequest.findAllSite();
		
		request.fire(new Receiver<List<SiteProxy>>() {
			@Override
			public void onSuccess(List<SiteProxy> listSites) {
				view.setListSites(listSites);
			}
		});
	}
	
	

}