package fr.janua.client.presenter;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.Window;
import com.google.web.bindery.event.shared.SimpleEventBus;
import com.google.web.bindery.requestfactory.shared.Receiver;
import com.google.web.bindery.requestfactory.shared.Request;
import com.google.web.bindery.requestfactory.shared.ServerFailure;
import com.mvp4g.client.annotation.Presenter;
import com.mvp4g.client.presenter.LazyPresenter;

import fr.janua.client.CrudRequestFactory;
import fr.janua.client.JaguardsEventBus;
import fr.janua.client.UserManager;
import fr.janua.client.internationalization.JaguardsConstants;
import fr.janua.client.proxy.MessageProxy;
import fr.janua.client.proxy.SiteProxy;
import fr.janua.client.proxy.TypeProxy;
import fr.janua.client.proxy.UserProxy;
import fr.janua.client.stub.MessageRequest;
import fr.janua.client.stub.UserRequest;
import fr.janua.client.view.PageMainCouranteView;
import fr.janua.client.view.interfaces.IPageMainCouranteView;
import fr.janua.client.view.interfaces.IPageMainCouranteView.IPageMainCourantePresenter;
import fr.janua.shared.Action;


@Presenter( view = PageMainCouranteView.class )
public class PageMainCourantePresenter extends LazyPresenter<IPageMainCouranteView, JaguardsEventBus> implements IPageMainCourantePresenter {

	private JaguardsConstants constants = GWT.create(JaguardsConstants.class);
	private CrudRequestFactory requestFactory = GWT.create(CrudRequestFactory.class);
	
	private int NB_ELEM_PER_PAGE = 4;	// le nombre de message qu'on va afficher pour chaque page
	private int currentPageOpenMessage;
	private int currentPageClosedMessage;
	private int nbOpenMessage;
	private int nbClosedMessage;


	public void onGoToPageMainCourante(){
		eventBus.setBody( view );
		view.loadPage();
	}



	/* Override */

	@Override
	public void createMessage(final String title, final String description, final SiteProxy site, final Set<TypeProxy> types) {
		// 1 - on récupère le login
		String login = UserManager.getInstance().getLogin();

		if (requestFactory.getEventBus() == null) {
			requestFactory.initialize(new SimpleEventBus());
		}
		// 2 - on récupère le UserProxy à partir de son login (utile pour avoir avoir un group à jour)
		UserRequest userRequest = requestFactory.getUserRequest();
		Request<UserProxy> request = userRequest.findByLogin(login).with("group", "siteAllowed");
		request.fire(new Receiver<UserProxy>() {
			@Override
			public void onSuccess(UserProxy userProxy) {
				// 3 - on créer le message
				MessageRequest messageRequest = requestFactory.getMessageRequest();

				MessageProxy messageProxy = messageRequest.create(MessageProxy.class);
				messageProxy.setTitle(title);
				messageProxy.setDescription(description);
				messageProxy.setStatus(1);
				messageProxy.setGroupCreator(userProxy.getGroup().getName());
				messageProxy.setNameCreator(userProxy.getLogin());
				messageProxy.setSite(site);
				messageProxy.setTypes(types);

				Request<Boolean> requestGroup = messageRequest.create(messageProxy);
				requestGroup.fire(new Receiver<Boolean>() {
					@Override
					public void onSuccess(Boolean response) {
						// 4 - Si c'est bon, on efface les champs, et on actualise la liste des messages ouverts
						if(response){
							eventBus.addLog(Action.CREATE_MESSAGE, "succès [titre:" + title + "]");
							// Efface les champs "titre", "description", et remet au choix par défaut le "site" et le "type".
							view.resetFieldCreateMessage();
							// reload open message
							getNbOpenMessage();
							// redirection vers "Messages ouverts"
							view.goToTabOpenMessage();
							// Notification client
							Window.alert(constants.successCreateMessage());
						}else{
							eventBus.addLog(Action.CREATE_MESSAGE, "échec [title:" + title + "]");
							Window.alert(constants.failCreateMessage());
						}
					}
				});
			}
		});
	}

	@Override
	public void closeMessage(final MessageProxy message, final String comment) {
		if (requestFactory.getEventBus() == null) {
			requestFactory.initialize(new SimpleEventBus());
		}

		final MessageRequest messageRequest1 = requestFactory.getMessageRequest();
		// 1 - on récupère le MessageProxy
		final Request<MessageProxy> request = messageRequest1.findById(message.getId()).with("site", "types", "comments");
		request.fire(new Receiver<MessageProxy>() {
			@Override
			public void onSuccess(final MessageProxy messageProxy) {
				// 2 - on enlève le frozen de response, et on crée un 2nd context appelé messageRequest2 (messageRequest1 étant occupé)
				MessageRequest messageRequest2 = requestFactory.getMessageRequest();
				MessageProxy messageUpdated = messageRequest2.edit(messageProxy);
				// 3 - on modifie le MessageProxy récupéré dans la BDD
				messageUpdated.setCommentClosure(comment);
				messageUpdated.setNameCloser(UserManager.getInstance().getLogin());
				messageUpdated.setGroupCloser(UserManager.getInstance().getGroupName());
				messageUpdated.setStatus(0);
				// 4 - on le renvoie avec les nouvelle valeurs
				Request<Boolean> request2 = messageRequest2.update(messageUpdated);
				request2.fire(new Receiver<Boolean>() {
					@Override
					public void onSuccess(Boolean response) {
						if(response){
							// 5 - génération du pdf
							final MessageRequest messageRequest3 = requestFactory.getMessageRequest();
							// 1 - on récupère le MessageProxy
							final Request<Boolean> request2 = messageRequest3.generatePdf(messageProxy.getId());
							request2.fire(new Receiver<Boolean>() {
								@Override
								public void onSuccess(Boolean response) {
									if(!response)
										Window.alert(constants.failGeneratePdf());
								}});
							
							eventBus.addLog(Action.CLOSE_MESSAGE, "succès [id:" + message.getId() + "]");
							// reload open message
							getNbOpenMessage();
							// reload closed message
							getNbClosedMessage();
							// reload search
							view.refreshSearch();
							// Notification client
							Window.alert(constants.successClosureMessage());
						}else{
							eventBus.addLog(Action.CLOSE_MESSAGE, "échec [id:" + message.getId() + "]");
							Window.alert(constants.failClosureMessage());
						}
					}
				});
			}
			@Override
			public void onFailure(ServerFailure error) {
				Window.alert("error : " + error);
			}
		});

	}

	@Override
	public void getNbOpenMessage() {
		if(requestFactory.getEventBus() == null){
			requestFactory.initialize(new SimpleEventBus());
		}

		ArrayList<String> listSites = new ArrayList<String>();
		for(SiteProxy site : UserManager.getInstance().getSitesAllowed()){
			listSites.add(site.getName());
		}
		ArrayList<String> listTypes = new ArrayList<String>();
		for(TypeProxy type : UserManager.getInstance().getTypesAllowed()){
			listTypes.add(type.getName());
		}
		
		MessageRequest messageRequest = requestFactory.getMessageRequest();
		Request<Integer> request = messageRequest.getOpenMessageCount(listSites, listTypes);
		
		request.fire(new Receiver<Integer>() {
			@Override
			public void onSuccess(Integer response) {
				getOpenMessage(0, NB_ELEM_PER_PAGE);
				
				// Nombre de Messages ouverts total
				int nbMessage = response;
				// nombre de page à afficher
				int nbPage = getNbPage(nbMessage, NB_ELEM_PER_PAGE);
				// Saugevarde du nombre de page total
				nbOpenMessage = nbMessage;
				// page courante par défaut : page 1
				currentPageOpenMessage = 1;
				// Affiche la pagination
				view.displayPaginationOpenMessage(currentPageOpenMessage, nbPage);
				view.setNbOpenMessage(response);
			}
		});
	}

	@Override
	public void getNbClosedMessage() {
		if(requestFactory.getEventBus() == null){
			requestFactory.initialize(new SimpleEventBus());
		}
		
		ArrayList<String> listSites = new ArrayList<String>();
		for(SiteProxy site : UserManager.getInstance().getSitesAllowed()){
			listSites.add(site.getName());
		}
		ArrayList<String> listTypes = new ArrayList<String>();
		for(TypeProxy type : UserManager.getInstance().getTypesAllowed()){
			listTypes.add(type.getName());
		}

		MessageRequest messageRequest = requestFactory.getMessageRequest();
		Request<Integer> request = messageRequest.getClosedMessageCount(listSites, listTypes);
		
		request.fire(new Receiver<Integer>() {
			@Override
			public void onSuccess(Integer response) {
				view.setNbClosedMessage(response);
				getClosedMessage(0, NB_ELEM_PER_PAGE);
				
				// Nombre de Messages fermés total
				int nbMessage = response;
				// nombre de page à afficher
				int nbPage = getNbPage(nbMessage, NB_ELEM_PER_PAGE);
				// Saugevarde du nombre de page total
				nbClosedMessage = nbMessage;
				// page courante par défaut : page 1
				currentPageClosedMessage = 1;
				// Affiche la pagination
				view.displayPaginationClosedMessage(currentPageClosedMessage, nbPage);
			}
		});
	}

	@Override
	public void addComment(final MessageProxy message, String comment) {
		String login = UserManager.getInstance().getLogin();
		String groupName = UserManager.getInstance().getGroupName();
		if(requestFactory.getEventBus() == null){
			requestFactory.initialize(new SimpleEventBus());
		}

		MessageRequest messageRequest = requestFactory.getMessageRequest();
		Request<Boolean> request = messageRequest.addComment(message.getId(), comment, login, groupName);

		request.fire(new Receiver<Boolean>() {
			@Override
			public void onSuccess(Boolean response) {
				if(response){
					eventBus.addLog(Action.ADD_COMMENT, "succès [id:" + message.getId() + "]");
					// reload open message
					getNbOpenMessage();
					// reload search
					view.refreshSearch();
					// Notification client
					Window.alert(constants.successAddComment());
				}else{
					eventBus.addLog(Action.ADD_COMMENT, "échec [id:" + message.getId() + "]");
					Window.alert(constants.failAddComment());
				}
			}
		});
	}
	
	@Override
	public void addType(final MessageProxy message, final String typeName) {
		String login = UserManager.getInstance().getLogin();
		String groupName = UserManager.getInstance().getGroupName();
		
		if(requestFactory.getEventBus() == null){
			requestFactory.initialize(new SimpleEventBus());
		}
		
		TypeProxy type = findTypeByName(typeName);

		MessageRequest messageRequest = requestFactory.getMessageRequest();
		Request<Boolean> request = messageRequest.addType(message.getId(), type, login, groupName);

		request.fire(new Receiver<Boolean>() {
			@Override
			public void onSuccess(Boolean response) {
				if(response){
					eventBus.addLog(Action.UPDATE_MESSAGE, "succès [id:" + message.getId() + ", type:" + typeName + "]");
					// reload open message
					getNbOpenMessage();
					// reload search
					view.refreshSearch();
					// Notification client
					Window.alert(constants.successAddType());
				}else{
					eventBus.addLog(Action.UPDATE_MESSAGE, "échec [id:" + message.getId() + ", type:" + typeName + "]");
					Window.alert(constants.failAddType());
				}
			}
		});
	}
	
	@Override
	public void getProfile(final String nameCreator) {
		if(requestFactory.getEventBus() == null){
			requestFactory.initialize(new SimpleEventBus());
		}
		
		UserRequest userRequest = requestFactory.getUserRequest();
		Request<UserProxy> request = userRequest.findByLogin(nameCreator).with("group", "siteAllowed");

		request.fire(new Receiver<UserProxy>() {
			@Override
			public void onSuccess(UserProxy user) {
				if(user != null){
					eventBus.addLog(Action.READ_USER, "succès get info [login:" + nameCreator + "]");
					view.displayProfileInModal(user);
				}else{
					eventBus.addLog(Action.READ_USER, "échec get info [login:" + nameCreator + "]");
					Window.alert(constants.userNotFound());
				}
			}
		});
		
	}
	
	@Override
	public void getAllLoginUser() {
		if(requestFactory.getEventBus() == null){
			requestFactory.initialize(new SimpleEventBus());
		}
		
		UserRequest userRequest = requestFactory.getUserRequest();
		Request<List<String>> request = userRequest.findAllLoginUser();

		request.fire(new Receiver<List<String>>() {
			@Override
			public void onSuccess(List<String> listLogins) {
				view.setListLogin(listLogins);
				
			}
		});
		
	}
	
	@Override
	public void updateLastActivity() {
		if(requestFactory.getEventBus() == null){
			requestFactory.initialize(new SimpleEventBus());
		}
		
		UserRequest userRequest = requestFactory.getUserRequest();
		Request<Boolean> request = userRequest.updateLastActivity(UserManager.getInstance().getLogin());
		request.fire(new Receiver<Boolean>() {
			@Override
			public void onSuccess(Boolean result) {
				if (result){
					// Si la date a bien été actualisé, on actualise notre utilisateur courant
					// La page va ensuite se recharger toute seule
					eventBus.refreshLastActivityUser();
				}
			}
		});
	}
	

	@Override
	public void getOpenMessagePage(int numPage) {
		int nbPage = getNbPage(nbOpenMessage, NB_ELEM_PER_PAGE);
		currentPageOpenMessage = numPage;
		getOpenMessage((currentPageOpenMessage-1)*NB_ELEM_PER_PAGE, NB_ELEM_PER_PAGE);
		view.displayPaginationOpenMessage(currentPageOpenMessage, nbPage);
	}
	@Override
	public void previousPageOpenMessage() {
		int nbPage = getNbPage(nbOpenMessage, NB_ELEM_PER_PAGE);
		if(currentPageOpenMessage > 1)
			currentPageOpenMessage--;
		getOpenMessage((currentPageOpenMessage-1)*NB_ELEM_PER_PAGE, NB_ELEM_PER_PAGE);
		view.displayPaginationOpenMessage(currentPageOpenMessage, nbPage);
	}
	@Override
	public void nextPageOpenMessage() {
		int nbPage = getNbPage(nbOpenMessage, NB_ELEM_PER_PAGE);
		if(currentPageOpenMessage < nbPage)
			currentPageOpenMessage++;
		getOpenMessage((currentPageOpenMessage-1)*NB_ELEM_PER_PAGE, NB_ELEM_PER_PAGE);
		view.displayPaginationOpenMessage(currentPageOpenMessage, nbPage);
	}
	@Override
	public void lastPageOpenMessage() {
		int nbPage = getNbPage(nbOpenMessage, NB_ELEM_PER_PAGE);
		currentPageOpenMessage = nbPage;
		getOpenMessage((currentPageOpenMessage-1)*NB_ELEM_PER_PAGE, NB_ELEM_PER_PAGE);
		view.displayPaginationOpenMessage(currentPageOpenMessage, nbPage);
	}
	
	@Override
	public void getClosedMessagePage(int numPage) {
		int nbPage = getNbPage(nbClosedMessage, NB_ELEM_PER_PAGE);
		currentPageClosedMessage = numPage;
		getClosedMessage((currentPageClosedMessage-1)*NB_ELEM_PER_PAGE, NB_ELEM_PER_PAGE);
		view.displayPaginationClosedMessage(currentPageClosedMessage, nbPage);
	}
	@Override
	public void previousPageClosedMessage() {
		int nbPage = getNbPage(nbClosedMessage, NB_ELEM_PER_PAGE);
		if(currentPageClosedMessage > 1)
			currentPageClosedMessage--;
		getClosedMessage((currentPageClosedMessage-1)*NB_ELEM_PER_PAGE, NB_ELEM_PER_PAGE);
		view.displayPaginationClosedMessage(currentPageClosedMessage, nbPage);
	}
	@Override
	public void nextPageClosedMessage() {
		int nbPage = getNbPage(nbClosedMessage, NB_ELEM_PER_PAGE);
		if(currentPageClosedMessage < nbPage)
			currentPageClosedMessage++;
		getClosedMessage((currentPageClosedMessage-1)*NB_ELEM_PER_PAGE, NB_ELEM_PER_PAGE);
		view.displayPaginationClosedMessage(currentPageClosedMessage, nbPage);
	}
	@Override
	public void lastPageClosedMessage() {
		int nbPage = getNbPage(nbClosedMessage, NB_ELEM_PER_PAGE);
		currentPageClosedMessage = nbPage;
		getClosedMessage((currentPageClosedMessage-1)*NB_ELEM_PER_PAGE, NB_ELEM_PER_PAGE);
		view.displayPaginationClosedMessage(currentPageClosedMessage, nbPage);
	}

	
	
	/* Implémentation événement GWT */
	/**
	 * 
	 * Utilisé lorsqu'un message est correctement crée.
	 */
	public void onClearFieldCreateMessage(){
		
	}


	
	public void onReloadCloseMessage(){
		getNbClosedMessage();
	}

	
	/* Fonctions */

	private void getOpenMessage(final int startIndex, final int nbMessage) {
		if(requestFactory.getEventBus() == null){
			requestFactory.initialize(new SimpleEventBus());
		}

		
		ArrayList<String> listSites = new ArrayList<String>();
		for(SiteProxy site : UserManager.getInstance().getSitesAllowed()){
			listSites.add(site.getName());
		}
		ArrayList<String> listTypes = new ArrayList<String>();
		for(TypeProxy type : UserManager.getInstance().getTypesAllowed()){
			listTypes.add(type.getName());
		}
		
		
		MessageRequest messageRequest = requestFactory.getMessageRequest();
		Request<List<MessageProxy>> request = messageRequest.findOpenMessages(startIndex, nbMessage, listSites, listTypes).with("comments", "comments.type", "types", "site");
		
		request.fire(new Receiver<List<MessageProxy>>() {
			@Override
			public void onSuccess(List<MessageProxy> response) {
				eventBus.addLog(Action.READ_MESSAGE, "message ouverts [" + startIndex + " - " + (startIndex+nbMessage) + "]");
				displayOpenMessages(response);
			}
		});
	}
	private void getClosedMessage(final int startIndex, final int nbMessage) {
		if(requestFactory.getEventBus() == null){
			requestFactory.initialize(new SimpleEventBus());
		}
		
		ArrayList<String> listSites = new ArrayList<String>();
		for(SiteProxy site : UserManager.getInstance().getSitesAllowed()){
			listSites.add(site.getName());
		}
		ArrayList<String> listTypes = new ArrayList<String>();
		for(TypeProxy type : UserManager.getInstance().getTypesAllowed()){
			listTypes.add(type.getName());
		}

		MessageRequest messageRequest = requestFactory.getMessageRequest();
		Request<List<MessageProxy>> request = messageRequest.findClosedMessages(startIndex, nbMessage, listSites, listTypes).with("comments", "comments.type", "types", "site");
		
		request.fire(new Receiver<List<MessageProxy>>() {
			@Override
			public void onSuccess(List<MessageProxy> response) {
				eventBus.addLog(Action.READ_MESSAGE, "message fermés [" + startIndex + " - " + (startIndex+nbMessage) + "]");
				displayClosedMessages(response);
			}
		});
	}
	
	
	@Override
	public void search(String query) {
		if(requestFactory.getEventBus() == null){
			requestFactory.initialize(new SimpleEventBus());
		}
		MessageRequest messageRequest = requestFactory.getMessageRequest();
		Request<List<MessageProxy>> request = messageRequest.search(query).with("comments", "comments.type", "types", "site");
		request.fire(new Receiver<List<MessageProxy>>() {
			@Override
			public void onSuccess(List<MessageProxy> messages) {
				eventBus.addLog(Action.READ_MESSAGE, "succès [search:" + messages.size() + "results]");
				displayMessagesFound(messages);
			}
		});
	}



	/**
	 * Gère l'affichage des messages ouverts.
	 * @param listMessage
	 */
	private void displayOpenMessages(List<MessageProxy> listMessage){
		view.clearOpenMessage();
		for(MessageProxy m : listMessage){
			view.displayOneMessageInOpenMessage(m);
			view.displaySeparatorInOpenMessages();
		}
	}

	/**
	 * Gère l'affichage des messages fermés.
	 * @param listMessage
	 */
	private void displayClosedMessages(List<MessageProxy> listMessage){
		view.clearClosedMessage();
		for(MessageProxy m : listMessage){
			view.displayOneMessageInClosedMessage(m);
			view.displaySeparatorInClosedMessages();
		}
	}
	

	/**
	 * Gère l'affichage des messages résultant de la recherche.
	 * @param listMessage
	 */
	private void displayMessagesFound(List<MessageProxy> listMessage){
		view.clearMessagesFound();
		if(listMessage.size() == 0){
			view.noResultsFound();
		}else{
			for(MessageProxy m : listMessage){
				view.displayOneMessageInMessagesFound(m);
				view.displaySeparatorInMessagesFound();
			}
		}
	}



	/**
	 * Retourne le nombre de page que l'on doit afficher en fonction du nombre de Message et du nombre d'élément par page voulu.
	 * @param nbMessage
	 * @param nbElemPerPage
	 * @return
	 */
	private int getNbPage(int nbMessage, int nbElemPerPage){
		int nbPage;
		if(nbMessage%nbElemPerPage == 0)
			nbPage = (nbMessage/nbElemPerPage);
		else
			nbPage = (nbMessage/nbElemPerPage)+1;
		return nbPage;
	}


	private TypeProxy findTypeByName(String typeName){
		for(TypeProxy type : UserManager.getInstance().getTypesAllowed()){
			if(type.getName().equals(typeName)){
				return type;
			}
		}
		return null;
	}







	






	



	



	

	



	



	














}