package fr.janua.client.presenter;

import java.util.List;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.Window;
import com.google.web.bindery.event.shared.SimpleEventBus;
import com.google.web.bindery.requestfactory.shared.Receiver;
import com.google.web.bindery.requestfactory.shared.Request;
import com.mvp4g.client.annotation.Presenter;
import com.mvp4g.client.presenter.LazyPresenter;

import fr.janua.client.CrudRequestFactory;
import fr.janua.client.JaguardsEventBus;
import fr.janua.client.UserManager;
import fr.janua.client.internationalization.JaguardsConstants;
import fr.janua.client.proxy.TypeProxy;
import fr.janua.client.stub.TypeRequest;
import fr.janua.client.view.PageTypeView;
import fr.janua.client.view.interfaces.IPageTypeView;
import fr.janua.client.view.interfaces.IPageTypeView.IPageTypePresenter;
import fr.janua.shared.Action;


@Presenter( view = PageTypeView.class )
public class PageTypePresenter extends LazyPresenter<IPageTypeView, JaguardsEventBus> implements IPageTypePresenter {
	private JaguardsConstants constants = GWT.create(JaguardsConstants.class);
	private CrudRequestFactory requestFactory = GWT.create(CrudRequestFactory.class);

	public void onGoToPageType(){
		eventBus.setBody( view );
		view.loadPage();
	}

	@Override
	public void getAllType() {
		if(requestFactory.getEventBus() == null){
			requestFactory.initialize(new SimpleEventBus());
		}

		TypeRequest typeRequest = requestFactory.getTypeRequest();
		Request<List<TypeProxy>> request = typeRequest.findAllType();
		
		request.fire(new Receiver<List<TypeProxy>>() {
			@Override
			public void onSuccess(List<TypeProxy> response) {
				eventBus.addLog(Action.READ_Type, "[all]");
				for(TypeProxy type : response){
					view.displayOneType(type.getName(),type.getBgColor(), type.getFgColor(), type.isEnabled());
				}
			}
		});
	}

	@Override
	public void createType(final String name, String bgColor, String fgColor, boolean enabled) {
		if(requestFactory.getEventBus() == null){
			requestFactory.initialize(new SimpleEventBus());
		}
		
		TypeRequest typeRequest = requestFactory.getTypeRequest();
		
		TypeProxy typeProxy = typeRequest.create(TypeProxy.class);
		typeProxy.setName(name);
		typeProxy.setBgColor(bgColor);
		typeProxy.setFgColor(fgColor);
		typeProxy.setEnabled(enabled);
		
		Request<Boolean> request = typeRequest.create(typeProxy);
		request.fire(new Receiver<Boolean>() {
			@Override
			public void onSuccess(Boolean response) {
				if(response){
					eventBus.addLog(Action.CREATE_TYPE, "échec [name:" + name + "]");
					view.loadPage();
					Window.alert(constants.successCreateType());
				}else{
					eventBus.addLog(Action.CREATE_TYPE, "succès [name:" + name + "]");
					Window.alert(constants.failCreateType());
				}
			}
		});
	}

	@Override
	public void updateType(final String name, final String bgColor, final String fgColor, final boolean enabled) {
		if(requestFactory.getEventBus() == null){
			requestFactory.initialize(new SimpleEventBus());
		}
		final TypeRequest typeRequest1 = requestFactory.getTypeRequest();

		// 1 - on récupère le TypeProxy
		final Request<TypeProxy> request = typeRequest1.findByName(name);

		request.fire(new Receiver<TypeProxy>() {
			@Override
			public void onSuccess(TypeProxy response) {
				if(response == null) {
					eventBus.addLog(Action.UPDATE_TYPE, "échec, le type " + name + " n'existe pas");
					Window.alert(constants.typeDoesntExist());
				}else{
					// 2 - on enlève le frozen de response, et on crée un 2nd context appelé typeRequest2 (typeRequest1 étant occupé)
					TypeRequest typeRequest2 = requestFactory.getTypeRequest();
					TypeProxy typeUpdated = typeRequest2.edit(response);
	
					// 3 - on modifie le TypeProxy récupéré dans la BDD
					typeUpdated.setFgColor(fgColor);
					typeUpdated.setBgColor(bgColor);
					typeUpdated.setEnabled(enabled);
	
					// 4 - on le renvoie avec les nouvelle valeurs
					Request<Boolean> request2 = typeRequest2.update(typeUpdated);
	
					request2.fire(new Receiver<Boolean>() {
						@Override
						public void onSuccess(Boolean response) {
							if(response){
								for(TypeProxy type : UserManager.getInstance().getTypesAllowed()){
									if(type.getName().equals(name)){
										eventBus.refreshUserWithoutChangePassword();
										break;
									}
								}
								eventBus.addLog(Action.UPDATE_TYPE, "succès [name:" + name + "]");
								Window.alert(constants.successUpdateType());
								view.loadPage();
							}else{
								eventBus.addLog(Action.UPDATE_TYPE, "échec [name:" + name + "]");
								Window.alert(constants.failUpdateType());
							}
						}
					});
				}
			}
		});
	}




}