package fr.janua.client.presenter;

import java.util.List;
import java.util.Set;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.Window;
import com.google.web.bindery.event.shared.SimpleEventBus;
import com.google.web.bindery.requestfactory.shared.Receiver;
import com.google.web.bindery.requestfactory.shared.Request;
import com.mvp4g.client.annotation.Presenter;
import com.mvp4g.client.presenter.LazyPresenter;

import fr.janua.client.CrudRequestFactory;
import fr.janua.client.JaguardsEventBus;
import fr.janua.client.UserManager;
import fr.janua.client.internationalization.JaguardsConstants;
import fr.janua.client.proxy.GroupProxy;
import fr.janua.client.proxy.SiteProxy;
import fr.janua.client.proxy.UserProxy;
import fr.janua.client.stub.GroupRequest;
import fr.janua.client.stub.SiteRequest;
import fr.janua.client.stub.UserRequest;
import fr.janua.client.view.PageUserView;
import fr.janua.client.view.interfaces.IPageUserView;
import fr.janua.client.view.interfaces.IPageUserView.IPageUserPresenter;
import fr.janua.shared.Action;

@Presenter( view = PageUserView.class )
public class PageUserPresenter extends LazyPresenter<IPageUserView, JaguardsEventBus> implements IPageUserPresenter {
	private JaguardsConstants constants = GWT.create(JaguardsConstants.class);
	private CrudRequestFactory requestFactory = GWT.create(CrudRequestFactory.class);
	
	private final int NB_ELEM_PER_PAGE = 3;	// le nombre de user qu'on va afficher pour chaque page
	private int currentPage;
	private int nbUser;

	public void onGoToPageUser(){
		eventBus.setBody( view );
		view.loadPage();
	}

	@Override
	public void generatePagination() {
		if(requestFactory.getEventBus() == null){
			requestFactory.initialize(new SimpleEventBus());
		}
		UserRequest userRequest = requestFactory.getUserRequest();

		Request<Integer> request = userRequest.getUserCount();
		request.fire(new Receiver<Integer>() {
			@Override
			public void onSuccess(Integer result) {
				int nbPage = getNbPage(result, NB_ELEM_PER_PAGE);
				nbUser = result;
				currentPage = 1;
				view.displayPagination(currentPage, nbPage);
			}
		});
	}

	@Override
	public void createUser(final String login, final String firstName, final String lastName, final String mail, final String roleName, final String password, final Set<SiteProxy> siteAllowed, final boolean enabled) {
		if(requestFactory.getEventBus() == null){
			requestFactory.initialize(new SimpleEventBus());
		}

		// 1 - On recherche récupère le groupeProxy à partir de son nom (s'il n'existe pas, on arrête l'exécution de la requête, et on affiche une alert)
		GroupRequest groupRequest = requestFactory.getGroupRequest();
		Request<GroupProxy> request = groupRequest.findByName(roleName);
		request.fire(new Receiver<GroupProxy>() {
			@Override
			public void onSuccess(GroupProxy group) {
				if(group == null){
					eventBus.addLog(Action.CREATE_USER, "échec [login:" + login + "]");
					Window.alert(constants.failCreateUserGroupNotFound());
				}else{
					// 2 - Maintenant qu'on a le groupProxy, on l'ajoute va essayer de créer le nouvel utilisateur
					UserRequest userRequest = requestFactory.getUserRequest();

					UserProxy userProxy = userRequest.create(UserProxy.class);
					userProxy.setLogin(login);
					userProxy.setFirstName(firstName);
					userProxy.setLastName(lastName);
					userProxy.setMail(mail);
					userProxy.setGroup(group);
					userProxy.setPassword(password);
					userProxy.setSiteAllowed(siteAllowed);
					userProxy.setEnabled(enabled);
					userProxy.setShowHelp(true);	// par défaut, tous les utilisateurs ont l'aide d'activé

					Request<Boolean> request = userRequest.create(userProxy);
					request.fire(new Receiver<Boolean>() {
						@Override
						public void onSuccess(Boolean response) {
							if(response){
								eventBus.addLog(Action.CREATE_USER, "succès [login:" + login + "]");
								Window.alert(constants.successCreateUser());
								view.loadPage();
							}else{
								eventBus.addLog(Action.CREATE_USER, "échec [login:" + login + "]");
								Window.alert(constants.failCreateUser());
							}
						}
					});

				}
			}
		});
	}

	@Override
	public void updateUser(final String login, final String firstName, final String lastName, final String mail, final Set<SiteProxy> siteAllowed, final String groupName, final boolean enabled) {
		if(requestFactory.getEventBus() == null){
			requestFactory.initialize(new SimpleEventBus());
		}

		// 1 - on récupère le GroupProxy
		final GroupRequest groupRequest1 = requestFactory.getGroupRequest();
		final Request<GroupProxy> request = groupRequest1.findByName(groupName);
		request.fire(new Receiver<GroupProxy>() {
			@Override
			public void onSuccess(final GroupProxy group) {

				// 2 - s'il existe, on va récupérer le UserProxy à modifier
				if(group == null){
					eventBus.addLog(Action.UPDATE_USER, "échec [login:" + login + "]");
					Window.alert(constants.failUpdateUserGroupNotFound());
				}else{
					final UserRequest userRequest1 = requestFactory.getUserRequest();
					final Request<UserProxy> request = userRequest1.findByLogin(login);
					request.fire(new Receiver<UserProxy>() {
						@Override
						public void onSuccess(UserProxy user) {

							// 3 - s'il existe, on modifie le UserProxy, et on l'envoie
							if(user == null){
								eventBus.addLog(Action.UPDATE_USER, "échec [login:" + login + "]");
								Window.alert(constants.failUpdateUserUserNotFound());
							}else{
								UserRequest userRequest2 = requestFactory.getUserRequest();
								UserProxy userUpdated = userRequest2.edit(user);

								userUpdated.setLogin(login);
								userUpdated.setFirstName(firstName);
								userUpdated.setLastName(lastName);
								userUpdated.setMail(mail);
								userUpdated.setSiteAllowed(siteAllowed);
								userUpdated.setGroup(group);
								userUpdated.setPassword("");	// on met le password à "" pour qu'il ne soit pas updaté. @see fr.janua.server.dao.object.UserDAO#update(User user)
								if(!UserManager.getInstance().getLogin().equals(user.getLogin()))
									// si ce n'est pas l'utilisateur courant
									userUpdated.setEnabled(enabled);

								// 4 - on le renvoie avec les nouvelle valeurs
								Request<Boolean> request2 = userRequest2.update(userUpdated);
								request2.fire(new Receiver<Boolean>() {
									@Override
									public void onSuccess(Boolean response) {
										if(response){
											// si on a modifié l'utilisateur courant...
											if(UserManager.getInstance().getLogin().equals(login)){
												// ... on actualise le UserProxy de notre UserManager !
												eventBus.refreshUserWithoutChangePassword();
											}
											eventBus.addLog(Action.UPDATE_USER, "succès [login:" + login + "]");
											Window.alert(constants.successUpdateUser());
											view.loadPage();
										}else{
											eventBus.addLog(Action.UPDATE_USER, "échec [login:" + login + "]");
											Window.alert(constants.failUpdateUser());
										}
									}
								});

							}
						}
					});
				}
			}
		});
	}

	@Override
	public void updatePassword(final String login, final String password) {
		if(requestFactory.getEventBus() == null){
			requestFactory.initialize(new SimpleEventBus());
		}

		// 1 - On récupère le User par son login
		final UserRequest userRequest1 = requestFactory.getUserRequest();
		final Request<UserProxy> request = userRequest1.findByLogin(login);
		request.fire(new Receiver<UserProxy>() {
			@Override
			public void onSuccess(UserProxy user) {

				// 2 - s'il existe, on modifie le UserProxy, et on l'envoie
				if(user == null){
					eventBus.addLog(Action.UPDATE_USER, "échec [login:" + login + "]");
					Window.alert(constants.failUpdateUserUserNotFound());
				}else{
					UserRequest userRequest2 = requestFactory.getUserRequest();
					UserProxy userUpdated = userRequest2.edit(user);

					userUpdated.setPassword(password);

					// 3 - on le renvoie avec les nouvelle valeurs
					Request<Boolean> request2 = userRequest2.update(userUpdated);
					request2.fire(new Receiver<Boolean>() {
						@Override
						public void onSuccess(Boolean response) {
							if(response){
								// si on a modifié le mot de passe de l'utilisateur courant...
								if(UserManager.getInstance().getLogin().equals(login)){
									// ... on actualise le UserProxy de notre UserManager !
									eventBus.refreshUser(login, password);
								}
								eventBus.addLog(Action.UPDATE_USER, "succès [login:" + login + "]");
								Window.alert(constants.successUpdateUser());
							}else{
								eventBus.addLog(Action.UPDATE_USER, "échec [login:" + login + "]");
								Window.alert(constants.failUpdateUser());
							}
						}
					});

				}
			}
		});
	}
	
	@Override
	public void checkAvailability(String login) {
		if(requestFactory.getEventBus() == null){
			requestFactory.initialize(new SimpleEventBus());
		}

		// 1 - On récupère tous les Groupes
		UserRequest userRequest = requestFactory.getUserRequest();
		Request<Boolean> request = userRequest.checkAvailability(login);
		request.fire(new Receiver<Boolean>() {
			@Override
			public void onSuccess(Boolean isAvailable) {
				view.loginAvailable(isAvailable);
			}
		});

	}
	
	@Override
	public void getAllSitesAllRoleGetAllUser() {
		if(requestFactory.getEventBus() == null){
			requestFactory.initialize(new SimpleEventBus());
		}
		
		// 1 - On récupère tous les Sites
		SiteRequest siteRequest = requestFactory.getSiteRequest();
		Request<List<SiteProxy>> request = siteRequest.findAllSite();
		
		request.fire(new Receiver<List<SiteProxy>>() {
			@Override
			public void onSuccess(List<SiteProxy> listSites) {
				view.setListSite(listSites);
				// 2 - On récupère tous les Groupes
				GroupRequest groupRequest = requestFactory.getGroupRequest();
				Request<List<GroupProxy>> request = groupRequest.findAllGroup(); //.with("siteAllowed", "typeAllowed");
				request.fire(new Receiver<List<GroupProxy>>() {
					@Override
					public void onSuccess(final List<GroupProxy> listGroup) {
						// 3 - Maintenant qu'on a les Groupes, on les envoie à la vue puis on lance la récupération de n utilisateurs
						view.setListGroup(listGroup);
						getUser(0, NB_ELEM_PER_PAGE);
		
					}
				});
			}
		});
	}

	@Override
	public void previousPage() {
		int nbPage = getNbPage(nbUser, NB_ELEM_PER_PAGE);
		if(currentPage > 1)
			currentPage--;
		getUser((currentPage-1)*NB_ELEM_PER_PAGE, NB_ELEM_PER_PAGE);
		view.displayPagination(currentPage, nbPage);
	}

	@Override
	public void nextPage() {
		int nbPage = getNbPage(nbUser, NB_ELEM_PER_PAGE);
		if(currentPage < nbPage)
			currentPage++;
		getUser((currentPage-1)*NB_ELEM_PER_PAGE, NB_ELEM_PER_PAGE);
		view.displayPagination(currentPage, nbPage);
	}

	@Override
	public void getUserPage(int numPage) {
		int nbPage = getNbPage(nbUser, NB_ELEM_PER_PAGE);
		currentPage = numPage;
		getUser((currentPage-1)*NB_ELEM_PER_PAGE, NB_ELEM_PER_PAGE);
		view.displayPagination(currentPage, nbPage);
	}
	
	@Override
	public void lastPage() {
		int nbPage = getNbPage(nbUser, NB_ELEM_PER_PAGE);
		currentPage = nbPage;
		getUser((currentPage-1)*NB_ELEM_PER_PAGE, NB_ELEM_PER_PAGE);
		view.displayPagination(currentPage, nbPage);
	}
	/* Fonctions */

	/**
	 * Envoie une requête pour récupérer la liste de tous les Users.
	 * @param indexStart Premier élément récupéré
	 * @param nbElement nombre d'élément à récupérer
	 */
	private void getUser(final int indexStart, final int nbElement) {
		if(requestFactory.getEventBus() == null){
			requestFactory.initialize(new SimpleEventBus());
		}
		
		UserRequest userRequest = requestFactory.getUserRequest();
		Request<List<UserProxy>> request = userRequest.findUser(indexStart, nbElement).with("group", "group.siteAllowed", "siteAllowed");
		request.fire(new Receiver<List<UserProxy>>() {
			@Override
			public void onSuccess(List<UserProxy> listUser) {
				eventBus.addLog(Action.READ_USER, "[" + indexStart + " - " + (indexStart+nbElement) + "]");
				view.clearPanelUser();
				// 3 - On envoie ensuite les Users vers la vue pour les afficher
				for(UserProxy user : listUser){
					view.displayOneUser(user);
				}
			}
		});
	}

	/**
	 * Retourne le nombre de page que l'on doit afficher en fonction du nombre de User et du nombre d'élément par page voulu.
	 * @param nbUser
	 * @param nbUserPerPage
	 * @return
	 */
	private int getNbPage(int nbUser, int nbUserPerPage){
		int nbPage;
		if(nbUser%nbUserPerPage == 0)
			nbPage = (nbUser/nbUserPerPage);
		else
			nbPage = (nbUser/nbUserPerPage)+1;
		return nbPage;
	}

	

	


	








}