package fr.janua.client.view;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.ChangeEvent;
import com.google.gwt.event.dom.client.ChangeHandler;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.KeyUpEvent;
import com.google.gwt.event.dom.client.KeyUpHandler;
import com.google.gwt.uibinder.client.UiBinder;
import com.google.gwt.uibinder.client.UiField;
import com.google.gwt.uibinder.client.UiHandler;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.Anchor;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.CheckBox;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.ListBox;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.Widget;

import fr.janua.client.StringManager;
import fr.janua.client.UserManager;
import fr.janua.client.internationalization.JaguardsConstants;
import fr.janua.client.proxy.GroupProxy;
import fr.janua.client.proxy.SiteProxy;
import fr.janua.client.proxy.TypeProxy;
import fr.janua.client.view.interfaces.IPageGroupView;
import fr.janua.client.view.interfaces.IPageGroupView.IPageGroupPresenter;

/**
 * 
 * @author Gilles Phan
 * 
 */
public class PageGroupView extends ReverseCompositeView<IPageGroupPresenter> implements IPageGroupView {
	private JaguardsConstants constants = GWT.create(JaguardsConstants.class);
	private static PageGroupViewUiBinder uiBinder = GWT.create(PageGroupViewUiBinder.class);

	interface PageGroupViewUiBinder extends
	UiBinder<Widget, PageGroupView> {}

	public PageGroupView() {
		initWidget(uiBinder.createAndBindUi(this));
	}

	private ArrayList<String> listGroupName;
	private List<SiteProxy> listSites;
	private List<TypeProxy> listTypes;
	
	
	@UiField Label label_error;
	@UiField FlowPanel fPanel_listGroup;
	@UiField Anchor anchor_addGroup;
	@UiField Button button_validateGroup;
	
	// modal
	@UiField TextBox  tBox_groupName, tBox_groupMail;
	@UiField CheckBox cBox_userCreate,	cBox_messageCreate,	cBox_groupCreate,	cBox_siteCreate,	cBox_typeCreate;
	@UiField CheckBox cBox_userEdit,	cBox_messageEdit,	cBox_groupEdit,		cBox_siteEdit,		cBox_typeEdit,	cBox_uploadDocEdit;
	@UiField CheckBox cBox_userView,	cBox_messageView,	cBox_groupView,		cBox_siteView,		cBox_typeView,	cBox_logView; //, cBox_adminBDView;
	@UiField ListBox  lBox_sitesAllowed, lBox_typesAllowed;
	
	/* Override */

	@Override
	public void createView() {
		listGroupName = new ArrayList<String>();
		anchor_addGroup.getElement().setAttribute("data-toggle", "modal");
		button_validateGroup.getElement().setAttribute("data-dismiss", "modal");
	}

	@Override
	public void loadPage() {
		// Efface la liste des rôles
		clearFPanel();
		clearModal();

		// Ajout du message dans le title
		if(DOM.getElementById("pressCtrlForMultipleSelection-1") != null)
			DOM.getElementById("pressCtrlForMultipleSelection-1").setTitle(constants.holdCtrlToSelectMultiple());
		if(DOM.getElementById("pressCtrlForMultipleSelection-2") != null)
			DOM.getElementById("pressCtrlForMultipleSelection-2").setTitle(constants.holdCtrlToSelectMultiple());
		
		if(UserManager.getInstance().isCreatableGroup()){
			anchor_addGroup.setVisible(true);
		}else{
			anchor_addGroup.setVisible(false);
		}
		
		if(UserManager.getInstance().isViewableGroup()){
			// Si droits suffisant : [requete] tous les groupes
			label_error.setVisible(false);
			presenter.getAllGroup();
		}else{
			// si non, signaler qu'on n'a pas les droits nécessaires
			label_error.setVisible(true);
		}
	}

	@Override
	public void displayGroups(List<SiteProxy> listSite, List<TypeProxy> listType, List<GroupProxy> listGroup) {
		// et ensuite, on affiche les groupe un par un
		int nbGroups = listGroup.size();
		for (int i = 0; i < nbGroups; i++) {
			displayOneGroup(listSite, listType, listGroup.get(i));
		}

	}

	@Override
	public void setListTypes(List<TypeProxy> listTypes) {
		this.listTypes = listTypes;
		
		lBox_typesAllowed.clear();
		for(TypeProxy t : listTypes){
			if(t.isEnabled())
				lBox_typesAllowed.addItem(t.getName());
			else
				lBox_typesAllowed.addItem(t.getName() + " (" + constants.disabled() + ")", t.getName());
		}
	}

	@Override
	public void setListSites(List<SiteProxy> listSites) {
		this.listSites = listSites;
		
		lBox_sitesAllowed.clear();
		for(SiteProxy s : listSites){
			if(s.isEnabled())
				lBox_sitesAllowed.addItem(s.getName());
			else
				lBox_sitesAllowed.addItem(s.getName() + " (" + constants.disabled() + ")", s.getName());
		}
	}

	/* Fonctions */

	/**
	 * Efface tout le contenu du FlowPanel contenant les droits d'accès.
	 * Efface aussi l'ArrayList des nom de rôle
	 */
	private void clearFPanel() {
		listGroupName.clear();
		fPanel_listGroup.clear();
	}

	/**
	 * Réinitialise les champs du modal (les 2 TextBox sont vidés, les boutons sont décochés (sauf 2), et les ListBox sont décochés aussi).
	 * Note : Cette fonction ne vide pas les ListBox (de site et de type) de leurs contenus.
	 */
	private void clearModal(){
		tBox_groupName.setText("");
		tBox_groupMail.setText("");

		cBox_messageCreate.setValue(true);
		cBox_messageEdit.setValue(false);
		cBox_messageView.setValue(true);
		
		cBox_userCreate.setValue(false);
		cBox_userEdit.setValue(false);
		cBox_userView.setValue(false);


		cBox_groupCreate.setValue(false);
		cBox_groupEdit.setValue(false);
		cBox_groupView.setValue(false);

		cBox_siteCreate.setValue(false);
		cBox_siteEdit.setValue(false);
		cBox_siteView.setValue(false);

		cBox_typeCreate.setValue(false);
		cBox_typeEdit.setValue(false);
		cBox_typeView.setValue(false);

		cBox_logView.setValue(false);
		
		/*cBox_adminBDEdit.setValue(false);
		cBox_adminBDView.setValue(false);*/
		
	}

	/**
	 * Affiche le rôle passé en paramètre, et enregistre ajoute son nom dans l'ArrayList (permet de checker au moment de la création d'un nouveau rôle).
	 * @param group
	 */
	private void displayOneGroup(final List<SiteProxy> listSite, final List<TypeProxy> listType, final GroupProxy group) {
		// Set de Widget pour pouvoir itérer dessus pour ajouter le changeHandler qui va afficher le bouton "Mettre à jour"
		Set<Widget> widgets = new HashSet<Widget>(0);
		
		// ajout à la liste
		listGroupName.add(group.getName());

		// enveloppera les données d'un groupe
		FlowPanel fPanelGroup = new FlowPanel();
		fPanelGroup.addStyleName("bloc-info span8 offset2");
		
		// Le titre
		FlowPanel fPanelTitle = new FlowPanel();
		fPanelTitle.setStyleName("row-fluid");
		fPanelTitle.add(new HTML("<div class='title-light-silver gradient'>" + StringManager.encodeHtml(group.getName()) + "</div>"));

		// - - - - - - - - - - - - - - - - - -
		// La zone gauche+droite
		FlowPanel fPanelContentGroup = new FlowPanel();
		fPanelContentGroup.setStyleName("row-fluid padding-top-10");
		// Sur la partie gauche : la gestion des droits CRUD 
		FlowPanel fPanelLeftPart = new FlowPanel();
		fPanelLeftPart.setStyleName("span6 description-message separator-vertical");
		// Sur la partie droite : la gestion du mail, et des Site/Type
		FlowPanel fPanelRightPart = new FlowPanel();
		fPanelRightPart.setStyleName("span6 description-message");

		fPanelContentGroup.add(fPanelLeftPart);
		fPanelContentGroup.add(fPanelRightPart);
		
		// - - - - - - - - - - - - - - - - - -
		// "Messages"
		FlowPanel fPanelCheckBoxMessage = new FlowPanel();
		fPanelCheckBoxMessage.setStyleName("span8");

		final CheckBox cBoxCreateMessage = new CheckBox(constants.create());
		final CheckBox cBoxEditMessage = new CheckBox(constants.edit());
		final CheckBox cBoxViewMessage = new CheckBox(constants.view());

		widgets.add(cBoxCreateMessage);
		widgets.add(cBoxEditMessage);
		widgets.add(cBoxViewMessage);
		
		cBoxCreateMessage.addStyleName("inline span4");
		cBoxEditMessage.addStyleName("inline span4");
		cBoxViewMessage.addStyleName("inline span4");
		
		if(! UserManager.getInstance().isEditableGroup()){
			cBoxCreateMessage.setEnabled(false);
			cBoxEditMessage.setEnabled(false);
			cBoxViewMessage.setEnabled(false);
		}

		if (group.getCrudMessage() > 3)
			cBoxCreateMessage.setValue(true);
		if (group.getCrudMessage() > 5 || group.getCrudMessage() == 2 || group.getCrudMessage() == 3)
			cBoxEditMessage.setValue(true);
		if ((group.getCrudMessage())%2 == 1)
			cBoxViewMessage.setValue(true);

		fPanelCheckBoxMessage.add(cBoxCreateMessage);
		fPanelCheckBoxMessage.add(cBoxEditMessage);
		fPanelCheckBoxMessage.add(cBoxViewMessage);
		
		// - - - - - - - - - - - - - - - - - -
		// "Profils"
		FlowPanel fPanelCheckBoxProfil = new FlowPanel();
		fPanelCheckBoxProfil.setStyleName("span8");
		
		final CheckBox cBoxCreateProfil = new CheckBox(constants.create());
		final CheckBox cBoxEditProfil = new CheckBox(constants.edit());
		final CheckBox cBoxViewProfil = new CheckBox(constants.view());

		widgets.add(cBoxCreateProfil);
		widgets.add(cBoxEditProfil);
		widgets.add(cBoxViewProfil);
		
		cBoxCreateProfil.addStyleName("inline span4");
		cBoxEditProfil.addStyleName("inline span4");
		cBoxViewProfil.addStyleName("inline span4");
		
		if(! UserManager.getInstance().isEditableGroup()){
			cBoxCreateProfil.setEnabled(false);
			cBoxEditProfil.setEnabled(false);
			cBoxViewProfil.setEnabled(false);
		}

		if (group.getCrudUser() > 3)
			cBoxCreateProfil.setValue(true);
		if (group.getCrudUser() > 5 || group.getCrudUser() == 2 || group.getCrudUser() == 3)
			cBoxEditProfil.setValue(true);
		if ((group.getCrudUser())%2 == 1)
			cBoxViewProfil.setValue(true);

		fPanelCheckBoxProfil.add(cBoxCreateProfil);
		fPanelCheckBoxProfil.add(cBoxEditProfil);
		fPanelCheckBoxProfil.add(cBoxViewProfil);

		// - - - - - - - - - - - - - - - - - -
		// "Groupes"
		FlowPanel fPanelCheckBoxGroup = new FlowPanel();
		fPanelCheckBoxGroup.setStyleName("span8");

		final CheckBox cBoxCreateGroup = new CheckBox(constants.create());
		final CheckBox cBoxEditGroup = new CheckBox(constants.edit());
		final CheckBox cBoxViewGroup = new CheckBox(constants.view());

		widgets.add(cBoxCreateGroup);
		widgets.add(cBoxEditGroup);
		widgets.add(cBoxViewGroup);
		
		cBoxCreateGroup.addStyleName("inline span4");
		cBoxEditGroup.addStyleName("inline span4");
		cBoxViewGroup.addStyleName("inline span4");
		
		if(! UserManager.getInstance().isEditableGroup()){
			cBoxCreateGroup.setEnabled(false);
			cBoxEditGroup.setEnabled(false);
			cBoxViewGroup.setEnabled(false);
		}

		if (group.getCrudGroup() > 3)
			cBoxCreateGroup.setValue(true);
		if (group.getCrudGroup() > 5 || group.getCrudGroup() == 2 || group.getCrudGroup() == 3)
			cBoxEditGroup.setValue(true);
		if ((group.getCrudGroup())%2 == 1)
			cBoxViewGroup.setValue(true);

		fPanelCheckBoxGroup.add(cBoxCreateGroup);
		fPanelCheckBoxGroup.add(cBoxEditGroup);
		fPanelCheckBoxGroup.add(cBoxViewGroup);

		// - - - - - - - - - - - - - - - - - -
		// "Location"
		FlowPanel fPanelCheckBoxLocation = new FlowPanel();
		fPanelCheckBoxLocation.setStyleName("span8");

		final CheckBox cBoxCreateLocation = new CheckBox(constants.create());
		final CheckBox cBoxEditLocation = new CheckBox(constants.edit());
		final CheckBox cBoxViewLocation = new CheckBox(constants.view());
		
		widgets.add(cBoxCreateLocation);
		widgets.add(cBoxEditLocation);
		widgets.add(cBoxViewLocation);
		
		cBoxCreateLocation.addStyleName("inline span4");
		cBoxEditLocation.addStyleName("inline span4");
		cBoxViewLocation.addStyleName("inline span4");
		
		if(! UserManager.getInstance().isEditableGroup()){
			cBoxCreateLocation.setEnabled(false);
			cBoxEditLocation.setEnabled(false);
			cBoxViewLocation.setEnabled(false);
		}

		if (group.getCrudSite() > 3)
			cBoxCreateLocation.setValue(true);
		if (group.getCrudSite() > 5 || group.getCrudSite() == 2 || group.getCrudSite() == 3)
			cBoxEditLocation.setValue(true);
		if ((group.getCrudSite())%2 == 1)
			cBoxViewLocation.setValue(true);

		fPanelCheckBoxLocation.add(cBoxCreateLocation);
		fPanelCheckBoxLocation.add(cBoxEditLocation);
		fPanelCheckBoxLocation.add(cBoxViewLocation);


		// - - - - - - - - - - - - - - - - - -
		// "Type"
		FlowPanel fPanelCheckBoxType = new FlowPanel();
		fPanelCheckBoxType.setStyleName("span8");

		final CheckBox cBoxCreateType = new CheckBox(constants.create());
		final CheckBox cBoxEditType = new CheckBox(constants.edit());
		final CheckBox cBoxViewType = new CheckBox(constants.view());

		widgets.add(cBoxCreateType);
		widgets.add(cBoxEditType);
		widgets.add(cBoxViewType);
		
		cBoxCreateType.addStyleName("inline span4");
		cBoxEditType.addStyleName("inline span4");
		cBoxViewType.addStyleName("inline span4");
		
		if(! UserManager.getInstance().isEditableGroup()){
			cBoxCreateType.setEnabled(false);
			cBoxEditType.setEnabled(false);
			cBoxViewType.setEnabled(false);
		}

		if (group.getCrudType() > 3)
			cBoxCreateType.setValue(true);
		if (group.getCrudType() > 5 || group.getCrudType() == 2 || group.getCrudType() == 3)
			cBoxEditType.setValue(true);
		if ((group.getCrudType())%2 == 1)
			cBoxViewType.setValue(true);

		fPanelCheckBoxType.add(cBoxCreateType);
		fPanelCheckBoxType.add(cBoxEditType);
		fPanelCheckBoxType.add(cBoxViewType);

		// - - - - - - - - - - - - - - - - - -
		// "Log"
		FlowPanel fPanelCheckBoxLog = new FlowPanel();
		fPanelCheckBoxLog.setStyleName("span8");

		final CheckBox cBoxViewLog = new CheckBox(constants.view());
		widgets.add(cBoxViewLog);
		cBoxViewLog.addStyleName("inline span4 offset8");
		if(! UserManager.getInstance().isEditableGroup())
			cBoxViewLog.setEnabled(false);
		if ((group.getCrudLog())%2 == 1)
			cBoxViewLog.setValue(true);

		fPanelCheckBoxLog.add(cBoxViewLog);

		// - - - - - - - - - - - - - - - - - -
		// "UploadDoc"
		FlowPanel fPanelCheckBoxUploadDoc = new FlowPanel();
		fPanelCheckBoxUploadDoc.setStyleName("span8");

		final CheckBox cBoxEditUploadDoc = new CheckBox(constants.edit());
		
		widgets.add(cBoxEditUploadDoc);
		
		cBoxEditUploadDoc.addStyleName("inline span4 offset4");
		
		if(! UserManager.getInstance().isEditableGroup()){
			cBoxEditUploadDoc.setEnabled(false);
		}

		if (group.getCrudUploadDoc() > 5 || group.getCrudUploadDoc() == 2 || group.getCrudUploadDoc() == 3)
			cBoxEditUploadDoc.setValue(true);

		fPanelCheckBoxUploadDoc.add(cBoxEditUploadDoc);
		
		// - - - - - - - - - - - - - - - - - -
		// "Mail"
		Label labelMail = new Label("Mail");
		labelMail.setStyleName("span3");

		final TextBox tboxMail = new TextBox();
		widgets.add(tboxMail);
		tboxMail.setText(group.getMail());
		tboxMail.addStyleName("span9");

		if(!UserManager.getInstance().isEditableGroup())
			tboxMail.setEnabled(false);
		
		FlowPanel fPanelMail = new FlowPanel();
		fPanelMail.addStyleName("row-fluid");
		fPanelMail.add(labelMail);
		fPanelMail.add(tboxMail);

		// - - - - - - - - - - - - - - - - - -
		// Liste déroulante : "Site"
		final ListBox lboxSite = new ListBox(true);
		widgets.add(lboxSite);
		lboxSite.addStyleName("span9");
		if(!UserManager.getInstance().isEditableGroup())
			lboxSite.setEnabled(false);
		
		int nbSiteSelected = 0;
		Set<SiteProxy> setSite = group.getSiteAllowed();
		final int nbSite = listSite.size();
		for(int i=0; i<nbSite; i++){
			if(listSite.get(i).isEnabled()){
				lboxSite.addItem(listSite.get(i).getName());
			}else{
				lboxSite.addItem(listSite.get(i).getName() + " (" + constants.disabled() + ")");
			}
			if(setSite.contains(listSite.get(i))){
				lboxSite.setItemSelected(i, true);
				nbSiteSelected++;
			}
		}
		
		HTML htmlLocation = new HTML(constants.locations() + " (" + nbSiteSelected + "/" + nbSite +") <i class='icon-info-sign help' title='" + constants.holdCtrlToSelectMultiple() + "'></i>");
		htmlLocation.setStyleName("span3");
		
		FlowPanel fPanelLocation = new FlowPanel();
		fPanelLocation.addStyleName("row-fluid");
		fPanelLocation.add(htmlLocation);
		fPanelLocation.add(lboxSite);

		// - - - - - - - - - - - - - - - - - -
		// Liste déroulante : "Type"
		final ListBox lboxType = new ListBox(true);
		widgets.add(lboxType);
		lboxType.addStyleName("span9");
		if(!UserManager.getInstance().isEditableGroup())
			lboxType.setEnabled(false);
		
		int nbTypeSelected = 0;
		Set<TypeProxy> setType = group.getTypeAllowed();
		final int nbType = listType.size();
		for(int i=0; i<nbType; i++){
			if(listType.get(i).isEnabled()){
				lboxType.addItem(listType.get(i).getName());
			}else{
				lboxType.addItem(listType.get(i).getName() + " (" + constants.hide() + ")");
			}
			if(setType.contains(listType.get(i))){
				lboxType.setItemSelected(i, true);
				nbTypeSelected++;
			}
		}

		HTML htmlType = new HTML("Type (" + nbTypeSelected + "/" + nbType +") <i class='icon-info-sign help' title='" + constants.holdCtrlToSelectMultiple() + "'></i>");
		htmlType.setStyleName("span3");
		
		FlowPanel fPanelType = new FlowPanel();
		fPanelType.addStyleName("row-fluid");
		fPanelType.add(htmlType);
		fPanelType.add(lboxType);

		// - - - - - - - - - - - - - - - - - -
		// Partie gauche
		fPanelLeftPart.add(new HTML("<div class='span4'>" + constants.messages() + "</div>"));
		fPanelLeftPart.add(fPanelCheckBoxMessage);
		fPanelLeftPart.add(new HTML("<div class='span4'>" + constants.profiles() + "</div>"));
		fPanelLeftPart.add(fPanelCheckBoxProfil);
		fPanelLeftPart.add(new HTML("<div class='span4'>" + constants.groups() + "</div>"));
		fPanelLeftPart.add(fPanelCheckBoxGroup);
		fPanelLeftPart.add(new HTML("<div class='span4'>" + constants.locations() + "</div>"));
		fPanelLeftPart.add(fPanelCheckBoxLocation);
		fPanelLeftPart.add(new HTML("<div class='span4'>" + constants.types() + "</div>"));
		fPanelLeftPart.add(fPanelCheckBoxType);
		fPanelLeftPart.add(new HTML("<div class='span4'>" + constants.logs() + "</div>"));
		fPanelLeftPart.add(fPanelCheckBoxLog);
		fPanelLeftPart.add(new HTML("<div class='span4'>" + constants.manageDocuments() + "</div>"));
		fPanelLeftPart.add(fPanelCheckBoxUploadDoc);
		// - - - - - - - - - - - - - - - - - -
		// Partie droite
		fPanelRightPart.add(fPanelMail);
		fPanelRightPart.add(fPanelLocation);
		fPanelRightPart.add(fPanelType);

		// - - - - - - - - - - - - - - - - - -
		// Bouton update
		FlowPanel fPanelMaj = null;
		final Button buttonUpdate = new Button(constants.update());
		if(UserManager.getInstance().isEditableGroup()){
			fPanelMaj = new FlowPanel();
			fPanelMaj.setStyleName("row-fluid padding-10");
			
			buttonUpdate.setVisible(false);
			buttonUpdate.addStyleName("btn btn-info");
			buttonUpdate.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					if(UserManager.getInstance().isEditableGroup()){
						String name = group.getName();
						String mail = tboxMail.getText();
	
						// droits sur les profils
						int numCrudUser = 0;
						if(cBoxCreateProfil.getValue()) numCrudUser += 4;
						if(cBoxEditProfil.getValue()) numCrudUser += 2;
						if(cBoxViewProfil.getValue()) numCrudUser += 1;
	
						// droits sur les messages
						int numCrudMessage = 0;
						if(cBoxCreateMessage.getValue()) numCrudMessage += 4;
						if(cBoxEditMessage.getValue()) numCrudMessage += 2;
						if(cBoxViewMessage.getValue()) numCrudMessage += 1;
	
						// droits sur les groupes
						int numCrudGroup = 0;
						if(cBoxCreateGroup.getValue()) numCrudGroup += 4;
						if(cBoxEditGroup.getValue()) numCrudGroup += 2;
						if(cBoxViewGroup.getValue()) numCrudGroup += 1;
	
						// droits sur les Sites
						int numCrudLocation = 0;
						if(cBoxCreateLocation.getValue()) numCrudLocation += 4;
						if(cBoxEditLocation.getValue()) numCrudLocation += 2;
						if(cBoxViewLocation.getValue()) numCrudLocation += 1;
	
						// droits sur les Types
						int numCrudType = 0;
						if(cBoxCreateType.getValue()) numCrudType += 4;
						if(cBoxEditType.getValue()) numCrudType += 2;
						if(cBoxViewType.getValue()) numCrudType += 1;
	
						// droits sur les Logs
						int numCrudLog = 0;
						if(cBoxViewLog.getValue()) numCrudLog += 1;
						
						// droits sur l'upload de fichiers
						int numCrudUploadDoc = 0;
						if(cBoxEditUploadDoc.getValue()) numCrudUploadDoc += 2;
	
						Set<SiteProxy> setSite = new HashSet<SiteProxy>();
						for(int i=0; i<nbSite; i++){
							if(lboxSite.isItemSelected(i)){
								setSite.add(listSite.get(i));
							}
						}
						
						Set<TypeProxy> setType = new HashSet<TypeProxy>();
						for(int i=0; i<nbType; i++){
							if(lboxType.isItemSelected(i)){
								setType.add(listType.get(i));
							}
						}
						
						// on le remasque
						buttonUpdate.setVisible(false);
	
						presenter.updateGroup(name, mail, setSite, setType, numCrudUser, numCrudMessage, numCrudGroup, numCrudLocation, numCrudType, numCrudLog, numCrudUploadDoc);
					}
	
				}
			});
			fPanelMaj.add(buttonUpdate);
		}
		// - - - - - - - - - - - - - - - - - -
		// Ajout du handler pour afficher le bouton "Mettre à jour"
		for(Widget w : widgets){
			w.sinkEvents(Event.ONCHANGE);
			w.addHandler(new ChangeHandler() {
				@Override
				public void onChange(ChangeEvent event) {
					buttonUpdate.setVisible(true);
				}
			}, ChangeEvent.getType());
		}
		tboxMail.addKeyUpHandler(new KeyUpHandler() {
			@Override
			public void onKeyUp(KeyUpEvent event) {
				buttonUpdate.setVisible(true);
			}
		});
		// - - - - - - - - - - - - - - - - - -
		fPanelGroup.add(fPanelTitle);
		fPanelGroup.add(fPanelContentGroup);
		if(UserManager.getInstance().isEditableGroup())
			fPanelGroup.add(fPanelMaj);
		
		FlowPanel fPanelWrapperGroup = new FlowPanel();
		fPanelWrapperGroup.addStyleName("row-fluid");
		fPanelWrapperGroup.add(fPanelGroup);
		
		fPanel_listGroup.add(fPanelWrapperGroup);

	}
	
	/**
	 * Retourne le SiteProxy en fonction de son nom.
	 * @param siteName
	 * @return
	 */
	private SiteProxy getSite(String siteName){
		for(SiteProxy s : listSites){
			if(s.getName().equals(siteName))
				return s;
		}
		return null;
	}
	
	/**
	 * Retourne le TypeProxy en fonction de son nom.
	 * @param typeName
	 * @return
	 */
	private TypeProxy getType(String typeName){
		for(TypeProxy t : listTypes){
			if(t.getName().equals(typeName))
				return t;
		}
		return null;
	}
	
	

	/* Handler */

	@UiHandler("button_validateGroup")
	void onClickValidateGroupCreation(ClickEvent e){
		if(UserManager.getInstance().isCreatableGroup()){
			// check si le champs "nom" n'est pas vide
			if(tBox_groupName.getText().equals("")){
				Window.alert(constants.pleaseEnterGroupName());
				return;
			}
			// check si ce nom n'existe pas déjà
			for(String name : listGroupName){
				if(name.equals(tBox_groupName.getText())){
					Window.alert(constants.groupNameAlreadyExists());
					return;
				}
			}
	
			String name = tBox_groupName.getText();
			String mail = tBox_groupMail.getText();
	
			int numCrudUser=0;
			if(cBox_userCreate.getValue()) numCrudUser+=4;
			if(cBox_userEdit.getValue()) numCrudUser+=2;
			if(cBox_userView.getValue()) numCrudUser+=1;
	
			int numCrudMessage=0;
			if(cBox_messageCreate.getValue()) numCrudMessage+=4;
			if(cBox_messageEdit.getValue()) numCrudMessage+=2;
			if(cBox_messageView.getValue()) numCrudMessage+=1;
	
			int numCrudGroup=0;
			if(cBox_groupCreate.getValue()) numCrudGroup+=4;
			if(cBox_groupEdit.getValue()) numCrudGroup+=2;
			if(cBox_groupView.getValue()) numCrudGroup+=1;
	
			int numCrudSite=0;
			if(cBox_siteCreate.getValue()) numCrudSite+=4;
			if(cBox_siteEdit.getValue()) numCrudSite+=2;
			if(cBox_siteView.getValue()) numCrudSite+=1;
	
			int numCrudType=0;
			if(cBox_typeCreate.getValue()) numCrudType+=4;
			if(cBox_typeEdit.getValue()) numCrudType+=2;
			if(cBox_typeView.getValue()) numCrudType+=1;
	
			int numCrudLog=0;
			if(cBox_logView.getValue()) numCrudLog+=1;
			
			int numCrudUploadDoc=0;
			if(cBox_uploadDocEdit.getValue()) numCrudUploadDoc+=2;
	
			Set<SiteProxy> setSite = new HashSet<SiteProxy>();
			int nbSite = lBox_sitesAllowed.getItemCount();
			for(int i=0; i<nbSite; i++)
				if(lBox_sitesAllowed.isItemSelected(i))
					setSite.add(getSite(lBox_sitesAllowed.getValue(i)));
			
			Set<TypeProxy> setType = new HashSet<TypeProxy>();
			int nbType = lBox_typesAllowed.getItemCount();
			for(int i=0; i<nbType; i++)
				if(lBox_typesAllowed.isItemSelected(i))
					setType.add(getType(lBox_typesAllowed.getValue(i)));

			presenter.createGroup(name, mail, setSite, setType, numCrudUser, numCrudMessage, numCrudGroup, numCrudSite, numCrudType, numCrudLog, numCrudUploadDoc);
		}
	}

	@UiHandler("anchor_addGroup")
	void onClickAddGroup(ClickEvent e){
		clearModal();
	}

	
}
