package fr.janua.client.view;

import com.google.gwt.core.client.GWT;
import com.google.gwt.dom.client.DivElement;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.KeyCodes;
import com.google.gwt.event.dom.client.KeyUpEvent;
import com.google.gwt.uibinder.client.UiBinder;
import com.google.gwt.uibinder.client.UiField;
import com.google.gwt.uibinder.client.UiHandler;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.Anchor;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.PasswordTextBox;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.UIObject;
import com.google.gwt.user.client.ui.Widget;

import fr.janua.client.internationalization.JaguardsConstants;
import fr.janua.client.view.interfaces.IPageLoginView;
import fr.janua.client.view.interfaces.IPageLoginView.IPageLoginPresenter;

/**
 * 
 * @author Gilles Phan
 * 
 */
public class PageLoginView extends ReverseCompositeView<IPageLoginPresenter> implements IPageLoginView {
	private JaguardsConstants constants = GWT.create(JaguardsConstants.class);
	private static PageLoginViewUiBinder uiBinder = GWT.create(PageLoginViewUiBinder.class);

	private boolean TEST_MODE = false;
	private boolean iswarningWisible = true;
	private int percentAdmin = 0;
	private int percentSite = 0;
	private int percentType = 0;
	private int percentAssoc = 0;

	interface PageLoginViewUiBinder extends UiBinder<Widget, PageLoginView> {
	}

	// Fields
	
	@UiField Anchor			 anchor_warningDB;
	@UiField TextBox		 tBox_login;
	@UiField PasswordTextBox ptBox_password;
	@UiField Button			 button_signIn;
	@UiField Button			 button_magicConnexion, button_init, button_createOtherSite, button_updateAdmin, btn_pull;
	@UiField DivElement		 div_progressbarSite, div_progressbarType, div_progressbarAdmin, div_progressbarAssoc;
	@UiField FlowPanel		 fPanel_progressSite, fPanel_progressType, fPanel_progressAdmin, fPanel_progressAssociation;

	
	public PageLoginView() {
		initWidget(uiBinder.createAndBindUi(this));
	}

	/* Override */

	@Override
	public void createView() {
		button_magicConnexion.setVisible(TEST_MODE);
		button_init.setVisible(TEST_MODE);
		button_createOtherSite.setVisible(TEST_MODE);
		button_updateAdmin.setVisible(TEST_MODE);
		UIObject.setVisible(div_progressbarSite, TEST_MODE);
		UIObject.setVisible(div_progressbarType, TEST_MODE);
		UIObject.setVisible(div_progressbarAdmin, TEST_MODE);
		UIObject.setVisible(div_progressbarAssoc, TEST_MODE);
		btn_pull.setVisible(TEST_MODE);
		
		anchor_warningDB.setTitle(constants.DBConfigFailClickToReload());
	}

	@Override
	public void loadPage() {
	}

	@Override
	public void clearField() {
		tBox_login.setText("");
		ptBox_password.setText("");
	}
	
	@Override
	public void setWarningVisible(boolean visible) {
		iswarningWisible = visible;
		anchor_warningDB.setVisible(visible);
	}

	

	@Override
	public void incrProgressSite() {
		percentSite+=17;
		fPanel_progressSite.setWidth(percentSite + "%");
	}
	@Override
	public void incrProgressType() {
		percentType+=15;
		fPanel_progressType.setWidth(percentType + "%");
	}
	@Override
	public void incrProgressAdmin() {
		percentAdmin+=50;
		fPanel_progressAdmin.setWidth(percentAdmin + "%");
	}
	@Override
	public void incrProgressAssoc() {
		percentAssoc+=100;
		fPanel_progressAssociation.setWidth(percentAssoc + "%");
	}

	/*
	 * Handlers
	 */

	@UiHandler("tBox_login")
	public void onKeyUpEnterLogin(KeyUpEvent e){
		if(e.getNativeKeyCode() == KeyCodes.KEY_ENTER)
			ptBox_password.setFocus(true);
	}
	@UiHandler("ptBox_password")
	public void onKeyUpEnterPassword(KeyUpEvent e){
		if(e.getNativeKeyCode() == KeyCodes.KEY_ENTER)
			presenter.checkSignIn(tBox_login.getText(), ptBox_password.getText());
	}

	@UiHandler("button_signIn")
	public void onClickButtonSignIn(ClickEvent e){
		presenter.checkSignIn(tBox_login.getText(), ptBox_password.getText());
	}

	@UiHandler("button_magicConnexion")
	public void onClickButtonMagicConnection(ClickEvent e){
		tBox_login.setText("admin");
		ptBox_password.setText("*");
		button_signIn.fireEvent(e);
	}

	@UiHandler("button_init")
	public void onClickButtonInit(ClickEvent e){
		tBox_login.setText("pompier1");
		ptBox_password.setText("*");
		button_signIn.fireEvent(e);
	}

	@UiHandler("button_createOtherSite")
	public void onClickButtonInitDbWithRequestFactory(ClickEvent e){
		Window.alert("Création de OtherSite !!");
		presenter.initDbStep1();
	}

	@UiHandler("button_updateAdmin")
	public void onClickButtonUpdateAdmin(ClickEvent e){
		presenter.initDbStep2();
	}

	@UiHandler("btn_pull")
	public void onClickBtnPull(ClickEvent e){
		String login = "admin";
		String password = "02197d6af3152be25ad528d65fe32317";
		String idMessage = "3";
		String namePDF = "hello.pdf";
		Window.Location.replace("/dl?"
				+ "login="+login
				+ "&password="+password
				+ "&idMessage="+idMessage
				+ "&namePDF="+namePDF);
	}

	
	@UiHandler("anchor_warningDB")
	public void onClickWarning(ClickEvent e){
		if(iswarningWisible){
			presenter.reloadDB();
		}
	}

}
