package fr.janua.client.view;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.uibinder.client.UiBinder;
import com.google.gwt.uibinder.client.UiField;
import com.google.gwt.uibinder.client.UiHandler;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.Anchor;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.Widget;

import fr.janua.client.StringManager;
import fr.janua.client.UserManager;
import fr.janua.client.internationalization.JaguardsConstants;
import fr.janua.client.view.interfaces.IPageSiteView;
import fr.janua.client.view.interfaces.IPageSiteView.IPageSitePresenter;

/**
 * 
 * @author Gilles Phan
 *
 */
public class PageSiteView extends ReverseCompositeView<IPageSitePresenter> implements IPageSiteView {
	private JaguardsConstants constants = GWT.create(JaguardsConstants.class);
	private static PageSiteViewUiBinder uiBinder = GWT.create(PageSiteViewUiBinder.class);


	interface PageSiteViewUiBinder extends UiBinder<Widget, PageSiteView> {}

	public PageSiteView() {
		initWidget(uiBinder.createAndBindUi(this));
	}

	@UiField Label label_error;
	
	@UiField FlowPanel fPanel_listSite;
	
	@UiField Anchor anchor_addSite;

	// modal add site
	@UiField TextBox tBox_nameSite;
	@UiField FlowPanel fPanel_showable;
	@UiField Button button_show, button_hide;
	@UiField Button button_valid;
	
	
	/* Override */

	@Override
	public void createView() {
		anchor_addSite.getElement().setAttribute("data-toggle", "modal");
		button_valid.getElement().setAttribute("data-dismiss", "modal");
		fPanel_showable.getElement().setAttribute("data-toggle", "buttons-radio");
	}

	@Override
	public void loadPage() {
		fPanel_listSite.clear();
		
		if(!UserManager.getInstance().isCreatableSite()){
			anchor_addSite.setVisible(false);
		}else{
			anchor_addSite.setVisible(true);
		}
		
		if(UserManager.getInstance().isViewableSite()){
			// Si droits suffisant
			label_error.setVisible(false);
			presenter.getAllSite();
		}else{
			// si non, signaler qu'on n'a pas les droits nécessaires
			label_error.setVisible(true);
		}
	}
	
	@Override
	public void displayOneSite(final String name, boolean enabled){
		final Button buttonUpdate = new Button(constants.update());
		
		FlowPanel fPanelSite = new FlowPanel();
		fPanelSite.setStyleName("span8 offset2 bloc-info");

		FlowPanel fPanelContent = new FlowPanel();
		fPanelContent.addStyleName("padding-top-10 padding-right-10 padding-left-10");
		
		// Nom du site
		String safeHtmlEnabled = "<span class='label label-success'>" + constants.enabled() + "</span>";
		if(! enabled)
			safeHtmlEnabled = "<span class='label label-warning'>" + constants.disabled() + "</span>";
		fPanelSite.add(new HTML("<div class='row-fluid'><div class='span12 title-light-silver gradient'>" + safeHtmlEnabled + " " + StringManager.encodeHtml(name) + "</div></div>"));
		
		// Etat
		Label labelState = new Label(constants.changeState());
		labelState.addStyleName("span3");
		
		final Button buttonEnabled = new Button(constants.enabled());
		Button buttonDisabled = new Button(constants.disabled());
		buttonEnabled.setStyleName("btn");
		buttonDisabled.setStyleName("btn");
		if(enabled)
			buttonEnabled.addStyleName("active");
		else
			buttonDisabled.addStyleName("active");
		buttonEnabled.addClickHandler(new ClickHandler() {
			@Override
			public void onClick(ClickEvent event) {
				buttonUpdate.setVisible(true);
			}
		});
		buttonDisabled.addClickHandler(new ClickHandler() {
			@Override
			public void onClick(ClickEvent event) {
				buttonUpdate.setVisible(true);
			}
		});
		FlowPanel wraperVisibility = new FlowPanel();
		wraperVisibility.setStyleName("row-fluid");
		
		FlowPanel fPanelWrapperButtonVisibility = new FlowPanel();
		fPanelWrapperButtonVisibility.addStyleName("btn-group");
		if(UserManager.getInstance().isEditableUser())
			fPanelWrapperButtonVisibility.getElement().setAttribute("data-toggle","buttons-radio");
		fPanelWrapperButtonVisibility.add(buttonEnabled);
		fPanelWrapperButtonVisibility.add(buttonDisabled);
		
		wraperVisibility.add(labelState);
		wraperVisibility.add(fPanelWrapperButtonVisibility);
		
		// Mettre à jour
		FlowPanel wraperUpdate = null;
		if(UserManager.getInstance().isEditableSite()){
			buttonUpdate.setVisible(false);
			buttonUpdate.setStyleName("btn btn-info");
			buttonUpdate.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					boolean isEnabledEdited = buttonEnabled.getStyleName().contains("active");
					buttonUpdate.setVisible(false);
					presenter.updateSite(name, isEnabledEdited);
				}
			});
			wraperUpdate = new FlowPanel();
			wraperUpdate.setStyleName("row-fluid padding-left-10 padding-bottom-10");
			wraperUpdate.add(buttonUpdate);
		}
		
		
		fPanelContent.add(wraperVisibility);
		
		fPanelSite.add(fPanelContent);
		if(UserManager.getInstance().isEditableSite())
			fPanelSite.add(wraperUpdate);
		
		FlowPanel fPanelWrapperSite = new FlowPanel();
		fPanelWrapperSite.addStyleName("row-fluid");
		fPanelWrapperSite.add(fPanelSite);
		
		fPanel_listSite.add(fPanelWrapperSite);
	}

	/* Fonction */
	
	// TODO : singleton
	/**
	 * Remplace les {<, >, -, --, \n} par les caractères spéciaux correspondant.
	 * @param text
	 * @return
	 */
	/*private String parseText(String text){
		String res = text;
		res = res.replace("--", "&#151;");
		res = res.replace("-", "&#150;");
		res = res.replace("<", "&lt;");
		res = res.replace(">", "&gt;");
		res = res.replace("\n", "<br/>");
		
		return res;
	}*/
	
	
	/* Handler */

	@UiHandler("anchor_addSite")
	public void onClickAddSite(ClickEvent e){
		// Réinitialise les valeurs par défaut
		tBox_nameSite.setText("");
		button_show.setStyleName("btn active");
		button_hide.setStyleName("btn");
	}
	
	@UiHandler("button_valid")
	public void onClickValid(ClickEvent e){
		if(UserManager.getInstance().isCreatableSite()){
			if(tBox_nameSite.getText().equals("")){
				Window.alert(constants.pleaseEnterSiteName());
			}else{
				// création site
				String name = tBox_nameSite.getText();
				boolean isEnabled = button_show.getStyleName().contains("active");
	
				presenter.createSite(name, isEnabled);
			}
		}
	}
}
