package fr.janua.client.view;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.ChangeEvent;
import com.google.gwt.event.dom.client.ChangeHandler;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.KeyUpEvent;
import com.google.gwt.event.dom.client.KeyUpHandler;
import com.google.gwt.uibinder.client.UiBinder;
import com.google.gwt.uibinder.client.UiField;
import com.google.gwt.uibinder.client.UiHandler;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.Anchor;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.Widget;

import fr.janua.client.StringManager;
import fr.janua.client.UserManager;
import fr.janua.client.internationalization.JaguardsConstants;
import fr.janua.client.view.interfaces.IPageTypeView;
import fr.janua.client.view.interfaces.IPageTypeView.IPageTypePresenter;

/**
 * 
 * @author Gilles Phan
 *
 */
public class PageTypeView extends ReverseCompositeView<IPageTypePresenter> implements IPageTypeView {
	private JaguardsConstants constants = GWT.create(JaguardsConstants.class);
	private static PageTypeViewUiBinder uiBinder = GWT.create(PageTypeViewUiBinder.class);


	interface PageTypeViewUiBinder extends UiBinder<Widget, PageTypeView> {}

	public PageTypeView() {
		initWidget(uiBinder.createAndBindUi(this));
	}

	@UiField Label label_error;
	
	@UiField FlowPanel fPanel_listType;
	
	@UiField Anchor anchor_addType;

	// modal add type
	@UiField TextBox tBox_nameType, tBox_bgColor;
	@UiField FlowPanel fPanel_fgColor, fPanel_showable;
	@UiField Button button_white, button_black, button_show, button_hide;
	@UiField Button button_valid;
	
	
	/* Override */

	@Override
	public void createView() {
		anchor_addType.getElement().setAttribute("data-toggle", "modal");
		button_valid.getElement().setAttribute("data-dismiss", "modal");
		fPanel_fgColor.getElement().setAttribute("data-toggle", "buttons-radio");
		fPanel_showable.getElement().setAttribute("data-toggle", "buttons-radio");
		tBox_bgColor.getElement().setAttribute("type", "color");
	}

	@Override
	public void loadPage() {
		fPanel_listType.clear();
		
		if(!UserManager.getInstance().isCreatableType()){
			anchor_addType.setVisible(false);
		}else{
			anchor_addType.setVisible(true);
		}
		
		if(UserManager.getInstance().isViewableType()){
			// Si droits suffisant
			label_error.setVisible(false);
			presenter.getAllType();
		}else{
			// si non, signaler qu'on n'a pas les droits nécessaires
			label_error.setVisible(true);
		}
	}
	
	@Override
	public void displayOneType(final String name, String bgColor, String textColor, boolean enabled){
		final Button buttonUpdate = new Button(constants.update());
		
		FlowPanel fPanelType = new FlowPanel();
		fPanelType.setStyleName("span8 offset2 bloc-info");

		FlowPanel fPanelContent = new FlowPanel();
		fPanelContent.addStyleName("padding-top-10 padding-right-10 padding-left-10");
		
		// Nom du type
		String safeHtmlEnabled = "<span class='label label-success'>" + constants.enabled() + "</span>";
		if(! enabled)
			safeHtmlEnabled = "<span class='label label-warning'>" + constants.disabled() + "</span>";
		fPanelType.add(new HTML("<div class='row-fluid'><div class='span12 title-light-silver gradient'>" + safeHtmlEnabled + " " + StringManager.encodeHtml(name) + "</div></div>"));
		
		// Couleur de fond
		Label labelBgColor = new Label(constants.backgroundColor());
		labelBgColor.addStyleName("span3");
		
		final TextBox tBoxBgColor = new TextBox();
		tBoxBgColor.getElement().setAttribute("type", "color");
		tBoxBgColor.setText(bgColor);
		tBoxBgColor.setStyleName("span9");
		if(!UserManager.getInstance().isEditableType())
			tBoxBgColor.setEnabled(false);
		tBoxBgColor.addKeyUpHandler(new KeyUpHandler() {
			@Override
			public void onKeyUp(KeyUpEvent event) {
				buttonUpdate.setVisible(true);
			}
		});
		tBoxBgColor.addChangeHandler(new ChangeHandler() {
			@Override
			public void onChange(ChangeEvent event) {
				buttonUpdate.setVisible(true);
			}
		});
		FlowPanel wraperBgColor = new FlowPanel();
		wraperBgColor.setStyleName("row-fluid");
		wraperBgColor.add(labelBgColor);
		wraperBgColor.add(tBoxBgColor);
		
		// Couleur de texte
		Label labelTextColor = new Label(constants.textColor());
		labelTextColor.addStyleName("span3");
		
		final Button buttonWhite = new Button(constants.white());
		Button buttonBlack = new Button(constants.black());
		buttonWhite.setStyleName("btn span6");
		buttonBlack.setStyleName("btn span6");
		if(textColor.equalsIgnoreCase("black"))
			buttonBlack.addStyleName("active");
		else
			buttonWhite.addStyleName("active");
		buttonWhite.addClickHandler(new ClickHandler() {
			@Override
			public void onClick(ClickEvent event) {
				buttonUpdate.setVisible(true);
			}
		});
		buttonBlack.addClickHandler(new ClickHandler() {
			@Override
			public void onClick(ClickEvent event) {
				buttonUpdate.setVisible(true);
			}
		});
		
		FlowPanel wraperButtonTextColor = new FlowPanel();
		wraperButtonTextColor.setStyleName("btn-group");
		if(UserManager.getInstance().isEditableType())
			wraperButtonTextColor.getElement().setAttribute("data-toggle","buttons-radio");
		wraperButtonTextColor.add(buttonWhite);
		wraperButtonTextColor.add(buttonBlack);
		
		FlowPanel wraperTextColor = new FlowPanel();
		wraperTextColor.add(labelTextColor);
		wraperTextColor.add(wraperButtonTextColor);
		
		
		// Etat
		Label labelState = new Label(constants.changeState());
		labelState.addStyleName("span3");
		
		final Button buttonEnabled = new Button(constants.enabled());
		Button buttonDisabled = new Button(constants.disabled());
		buttonEnabled.setStyleName("btn");
		buttonDisabled.setStyleName("btn");
		if(enabled)
			buttonEnabled.addStyleName("active");
		else
			buttonDisabled.addStyleName("active");
		buttonEnabled.addClickHandler(new ClickHandler() {
			@Override
			public void onClick(ClickEvent event) {
				buttonUpdate.setVisible(true);
			}
		});
		buttonDisabled.addClickHandler(new ClickHandler() {
			@Override
			public void onClick(ClickEvent event) {
				buttonUpdate.setVisible(true);
			}
		});
		FlowPanel wraperVisibility = new FlowPanel();
		wraperVisibility.setStyleName("row-fluid padding-top-10");
		
		FlowPanel fPanelWrapperButtonVisibility = new FlowPanel();
		fPanelWrapperButtonVisibility.addStyleName("btn-group");
		if(UserManager.getInstance().isEditableUser())
			fPanelWrapperButtonVisibility.getElement().setAttribute("data-toggle","buttons-radio");
		fPanelWrapperButtonVisibility.add(buttonEnabled);
		fPanelWrapperButtonVisibility.add(buttonDisabled);
		
		wraperVisibility.add(labelState);
		wraperVisibility.add(fPanelWrapperButtonVisibility);
		
		// Mettre à jour
		FlowPanel wraperUpdate = null;
		if(UserManager.getInstance().isEditableType()){
			buttonUpdate.setVisible(false);
			buttonUpdate.setStyleName("btn btn-info");
			buttonUpdate.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					boolean isEnabled = buttonEnabled.getStyleName().contains("active");
					buttonUpdate.setVisible(false);
					String bgColor = tBoxBgColor.getText();
					String textColor = (buttonWhite.getStyleName().contains("active")) ? "white" : "black";
					presenter.updateType(name, bgColor, textColor, isEnabled);
				}
			});
			wraperUpdate = new FlowPanel();
			wraperUpdate.setStyleName("row-fluid padding-left-10 padding-bottom-10");
			wraperUpdate.add(buttonUpdate);
		}
		
		fPanelContent.add(wraperBgColor);
		fPanelContent.add(wraperTextColor);
		fPanelContent.add(wraperVisibility);
		
		fPanelType.add(fPanelContent);
		if(UserManager.getInstance().isEditableType())
			fPanelType.add(wraperUpdate);
		
		FlowPanel fPanelWrapperType = new FlowPanel();
		fPanelWrapperType.addStyleName("row-fluid");
		fPanelWrapperType.add(fPanelType);
		
		fPanel_listType.add(fPanelWrapperType);
		
		/*final Button buttonUpdate = new Button("<i class='icon-hdd icon-white'></i>");
		
		FlowPanel wraperType = new FlowPanel();
		wraperType.setStyleName("row-fluid");
		
		// Nom du type
		TextBox tBoxNameType = new TextBox();
		tBoxNameType.setText(name);
		tBoxNameType.setStyleName("span12");
		tBoxNameType.setEnabled(false);
		FlowPanel wraperNameType = new FlowPanel();
		wraperNameType.setStyleName("span4");
		wraperNameType.add(tBoxNameType);
		
		// Couleur du fond
		final TextBox tBoxBgColor = new TextBox();
		tBoxBgColor.getElement().setAttribute("type", "color");
		tBoxBgColor.setText(bgColor);
		tBoxBgColor.setStyleName("span12");
		if(!UserManager.getInstance().isEditableType())
			tBoxBgColor.setEnabled(false);
//		tBoxBgColor.getElement().setAttribute("style", "background: " + bgColor + "; color: " + textColor);
		tBoxBgColor.addChangeHandler(new ChangeHandler() {
			@Override
			public void onChange(ChangeEvent event) {
				buttonUpdate.setVisible(true);
			}
		});
		FlowPanel wraperBgColor = new FlowPanel();
		wraperBgColor.setStyleName("span2");
		wraperBgColor.add(tBoxBgColor);
		
		// Couleur du texte
		Button buttonWhite = new Button(constants.white());
		final Button buttonBlack = new Button(constants.black());
		buttonWhite.setStyleName("btn span6");
		buttonBlack.setStyleName("btn span6");
		if(textColor.equalsIgnoreCase("black"))
			buttonBlack.addStyleName("active");
		else
			buttonWhite.addStyleName("active");
		buttonWhite.addClickHandler(new ClickHandler() {
			@Override
			public void onClick(ClickEvent event) {
				buttonUpdate.setVisible(true);
			}
		});
		buttonBlack.addClickHandler(new ClickHandler() {
			@Override
			public void onClick(ClickEvent event) {
				buttonUpdate.setVisible(true);
			}
		});
		FlowPanel wraperTextColor = new FlowPanel();
		wraperTextColor.setStyleName("span2 btn-group");
		if(UserManager.getInstance().isEditableType())
			wraperTextColor.getElement().setAttribute("data-toggle","buttons-radio");
		wraperTextColor.add(buttonWhite);
		wraperTextColor.add(buttonBlack);
		
		// Affichage
		final Button buttonEnable = new Button(constants.enabled());
		buttonEnable.setStyleName("btn span6");
		Button buttonDisable = new Button(constants.disabled());
		buttonDisable.setStyleName("btn span6");
		if(enabled){
			buttonEnable.addStyleName("active");
		}else{
			buttonDisable.addStyleName("active");
		}
		buttonEnable.addClickHandler(new ClickHandler() {
			@Override
			public void onClick(ClickEvent event) {
				buttonUpdate.setVisible(true);
			}
		});
		buttonDisable.addClickHandler(new ClickHandler() {
			@Override
			public void onClick(ClickEvent event) {
				buttonUpdate.setVisible(true);
			}
		});
		FlowPanel wraperVisibility = new FlowPanel();
		wraperVisibility.setStyleName("span2 btn-group");
		if(UserManager.getInstance().isEditableType())
			wraperVisibility.getElement().setAttribute("data-toggle","buttons-radio");
		wraperVisibility.add(buttonEnable);
		wraperVisibility.add(buttonDisable);
		
		// Mettre à jour
		FlowPanel wraperUpdate = null;
		if(UserManager.getInstance().isEditableType()){
			buttonUpdate.setVisible(false);
			buttonUpdate.setTitle(constants.update());
			buttonUpdate.setStyleName("btn btn-info span12");
			buttonUpdate.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					String currentName = name;
					String textColor = (buttonBlack.getStyleName().contains("active")) ? "black" : "whiteSmoke";
					String bgColor = tBoxBgColor.getText();
					boolean enabled = buttonEnable.getStyleName().contains("active");
					buttonUpdate.setVisible(false);
					presenter.updateType(currentName, bgColor, textColor, enabled);
				}
			});
			wraperUpdate = new FlowPanel();
			wraperUpdate.setStyleName("span2");
			wraperUpdate.add(buttonUpdate);
		}
		
		wraperType.add(wraperNameType);
		wraperType.add(wraperBgColor);
		wraperType.add(wraperTextColor);
		wraperType.add(wraperVisibility);
		if(UserManager.getInstance().isEditableType())
			wraperType.add(wraperUpdate);
		
		fPanel_listType.add(wraperType);
		*/
	}

	/* Fonction */

	// TODO : Singleton
	/**
	 * Remplace les {<, >, -, --, \n} par les caractères spéciaux correspondant.
	 * @param text
	 * @return
	 */
	/*private String parseText(String text){
		String res = text;
		res = res.replace("--", "&#151;");
		res = res.replace("-", "&#150;");
		res = res.replace("<", "&lt;");
		res = res.replace(">", "&gt;");
		res = res.replace("\n", "<br/>");
		
		return res;
	}*/
	
	
	/* Handler */

	@UiHandler("anchor_addType")
	public void onClickAddType(ClickEvent e){
		// Réinitialise les valeurs par défaut
		tBox_nameType.setText("");
		tBox_bgColor.setText("#777777");
		button_white.setStyleName("btn");
		button_black.setStyleName("btn active");
		button_show.setStyleName("btn active");
		button_hide.setStyleName("btn");
	}
	
	@UiHandler("button_valid")
	public void onClickValid(ClickEvent e){
		if(UserManager.getInstance().isCreatableType()){
			if(tBox_nameType.getText().equals("")){
				Window.alert(constants.pleaseEnterTypeName());
			}else if(tBox_bgColor.getText().equals("")){
				Window.alert(constants.pleaseEnterBackgroundColor());
			}else{
				// création type
				String nameType = tBox_nameType.getText();
				String bgColor = tBox_bgColor.getText();
				String fgColor = (button_white.getStyleName().contains("active")) ? "whiteSmoke" : "black";
				boolean enabled = button_show.getStyleName().contains("active");
				
				presenter.createType(nameType, bgColor, fgColor, enabled);
			}
		}
	}
	
}
