package fr.janua.client.view;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import com.google.gwt.core.client.GWT;
import com.google.gwt.dom.client.DivElement;
import com.google.gwt.dom.client.LIElement;
import com.google.gwt.event.dom.client.ChangeEvent;
import com.google.gwt.event.dom.client.ChangeHandler;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.KeyUpEvent;
import com.google.gwt.event.dom.client.KeyUpHandler;
import com.google.gwt.event.shared.HandlerRegistration;
import com.google.gwt.uibinder.client.UiBinder;
import com.google.gwt.uibinder.client.UiField;
import com.google.gwt.uibinder.client.UiHandler;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.Anchor;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.ListBox;
import com.google.gwt.user.client.ui.PasswordTextBox;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.UIObject;
import com.google.gwt.user.client.ui.Widget;

import fr.janua.client.StringManager;
import fr.janua.client.UserManager;
import fr.janua.client.internationalization.JaguardsConstants;
import fr.janua.client.proxy.GroupProxy;
import fr.janua.client.proxy.SiteProxy;
import fr.janua.client.proxy.UserProxy;
import fr.janua.client.view.interfaces.IPageUserView;
import fr.janua.client.view.interfaces.IPageUserView.IPageUserPresenter;

/**
 * 
 * @author Gilles Phan
 *
 */
public class PageUserView extends ReverseCompositeView<IPageUserPresenter> implements IPageUserView {
	private static PageUserViewUiBinder uiBinder = GWT.create(PageUserViewUiBinder.class);
	interface PageUserViewUiBinder extends UiBinder<Widget, PageUserView> {}

	private JaguardsConstants constants = GWT.create(JaguardsConstants.class);
	private String loginSelected; // lorsque l'on clique sur "changer" (de mot de passe), la valeur de "loginSelected" devient le login du User dont on veut changer le mot de passe

	private ArrayList<GroupProxy> listGroup;
	private ArrayList<SiteProxy> listSites;
	
	private Set<HandlerRegistration> handlerPagination;
	
	public PageUserView() {
		initWidget(uiBinder.createAndBindUi(this));
	}

	@UiField Label		label_error;
	@UiField Anchor		anchor_addUser;
	@UiField FlowPanel	fPanel_listUser;

	@UiField DivElement div_upPagination;	// up
	@UiField DivElement div_downPagination;	// down
	// nouvelle pagination : previous/next
	@UiField LIElement	li_upFirst,		li_upLast,		li_upPrevious,		li_upNext;		// up
	@UiField LIElement	li_downFirst,	li_downLast,	li_downPrevious,	li_downNext;	// down
	@UiField Anchor		anchor_upFirst,		anchor_upLast,		anchor_upPrevious,		anchor_upNext;		// up
	@UiField Anchor		anchor_cloneFirst,	anchor_cloneLast,	anchor_clonePrevious,	anchor_cloneNext;	// down
	// nouvelle pagination : pages A/B/C/D/E
	@UiField LIElement	li_upPageA,		li_upPageB,		li_upPageC,		li_upPageD,		li_upPageE;		// up
	@UiField LIElement	li_downPageA,	li_downPageB,	li_downPageC,	li_downPageD,	li_downPageE;	// down
	@UiField Anchor		anchor_upPageA,		anchor_upPageB,		anchor_upPageC,		anchor_upPageD,		anchor_upPageE;		// up
	@UiField Anchor		anchor_downPageA,	anchor_downPageB,	anchor_downPageC,	anchor_downPageD,	anchor_downPageE;	// down

	// modal "Ajout User"
	@UiField TextBox	tBox_loginUser, tBox_firstNameUser, tBox_lastNameUser, tBox_mailUser;
	@UiField FlowPanel	fPanel_loginDisponibility;
	@UiField ListBox	lBox_groupUser;
	@UiField PasswordTextBox ptBox_passwordUser1, ptBox_passwordUser2;
	@UiField ListBox	lBox_sitesAllowed;
	@UiField Button		button_enabled, button_disabled;
	@UiField Button		button_validAddUser;

	// modal "password"
	@UiField PasswordTextBox ptBox_changePassword1, ptBox_changePassword2;
	@UiField Button			 button_validNewPassword;

	/* Override */

	@Override
	public void createView() {
		handlerPagination = new HashSet<HandlerRegistration>();
		
		listGroup = new ArrayList<GroupProxy>();
		listSites = new ArrayList<SiteProxy>();
		anchor_addUser.getElement().setAttribute("data-toggle", "modal");
		button_validAddUser.getElement().setAttribute("data-dismiss", "modal");

		button_validNewPassword.getElement().setAttribute("data-dismiss", "modal");

		tBox_loginUser.addChangeHandler(new ChangeHandler() {
			@Override
			public void onChange(ChangeEvent event) {
				presenter.checkAvailability(tBox_loginUser.getText());
			}
		});
		
	}

	@Override
	public void loadPage() {
		// Ajout du message dans le title
		if(DOM.getElementById("pressCtrlForMultipleSelection") != null)
			DOM.getElementById("pressCtrlForMultipleSelection").setTitle(constants.holdCtrlToSelectMultiple());

		if(UserManager.getInstance().isCreatableUser()){
			anchor_addUser.setVisible(true);
		}else{
			anchor_addUser.setVisible(false);
		}
		if(UserManager.getInstance().isViewableUser()){
			// Si droits suffisant
			label_error.setVisible(false);
			// affiche la pagination
			UIObject.setVisible(div_upPagination, true);
			presenter.getAllSitesAllRoleGetAllUser();
			presenter.generatePagination();
		}else{
			// si non, signaler qu'on n'a pas les droits nécessaires
			//			button_reload.setVisible(false);
			label_error.setVisible(true);
			// masque la pagination
			UIObject.setVisible(div_upPagination, false);
		}
	}

	@Override
	public void displayOneUser(final UserProxy user){
		// Set de Widget pour pouvoir itérer dessus pour ajouter le changeHandler qui va afficher le bouton "Mettre à jour"
		Set<Widget> widgets = new HashSet<Widget>(0);
		
		final Button buttonUpdate = new Button(constants.update());

		FlowPanel fPanelUser = new FlowPanel();
		fPanelUser.setStyleName("span8 offset2 bloc-info");

		FlowPanel fPanelContent = new FlowPanel();
		fPanelContent.addStyleName("description-message");

		// Login et Activé/désactivé
		String safeHtmlEnabled = "<span class='label label-success'>" + constants.enabled() + "</span>";
		if(! user.isEnabled())
			safeHtmlEnabled = "<span class='label label-warning'>" + constants.disabled() + "</span>";
		fPanelUser.add(new HTML("<div class='row-fluid'><div class='span12 title-light-silver gradient'>" + safeHtmlEnabled + " " + StringManager.encodeHtml(user.getLogin()) + "</div></div>"));

		// Prénom
		Label labelFirstName = new Label(constants.firstName());
		labelFirstName.addStyleName("span3");

		final TextBox tBoxFirstName = new TextBox();
		tBoxFirstName.setText(user.getFirstName());
		tBoxFirstName.setStyleName("span9");
		if(!UserManager.getInstance().isEditableUser())
			tBoxFirstName.setEnabled(false);
		widgets.add(tBoxFirstName);
		FlowPanel wraperFirstName = new FlowPanel();
		wraperFirstName.setStyleName("row-fluid");
		wraperFirstName.add(labelFirstName);
		wraperFirstName.add(tBoxFirstName);

		// Nom
		Label labelLastName = new Label(constants.lastName());
		labelLastName.addStyleName("span3");

		final TextBox tBoxLastName = new TextBox();
		tBoxLastName.setText(user.getLastName());
		tBoxLastName.setStyleName("span9");
		if(!UserManager.getInstance().isEditableUser())
			tBoxLastName.setEnabled(false);
		widgets.add(tBoxLastName);
		FlowPanel wraperLastName = new FlowPanel();
		wraperLastName.setStyleName("row-fluid");
		wraperLastName.add(labelLastName);
		wraperLastName.add(tBoxLastName);

		// Mail
		Label labelMail = new Label(constants.mail());
		labelMail.addStyleName("span3");

		final TextBox tBoxMail = new TextBox();
		tBoxMail.setText(user.getMail());
		tBoxMail.setStyleName("span9");
		if(!UserManager.getInstance().isEditableUser())
			tBoxMail.setEnabled(false);
		widgets.add(tBoxMail);
		FlowPanel wraperMail = new FlowPanel();
		wraperMail.setStyleName("row-fluid");
		wraperMail.add(labelMail);
		wraperMail.add(tBoxMail);

		// Groupe
		Label labelGroup = new Label(constants.group());
		labelGroup.addStyleName("span3");

		final ListBox lBoxGroup = new ListBox();
		lBoxGroup.setStyleName("span9");
		for(int i=0; i<listGroup.size(); i++){
			lBoxGroup.addItem(listGroup.get(i).getName());
			if(listGroup.get(i).getName().equals(user.getGroup().getName())){
				lBoxGroup.setSelectedIndex(i);
			}
		}
		if(!UserManager.getInstance().isEditableUser())
			lBoxGroup.setEnabled(false);
		widgets.add(lBoxGroup);
		FlowPanel wraperGroup = new FlowPanel();
		wraperGroup.setStyleName("row-fluid");
		wraperGroup.add(labelGroup);
		wraperGroup.add(lBoxGroup);


		// Sites
		HTML htmlSitesAllowed = new HTML(constants.locations() + " <i class='icon-info-sign help' title='" + constants.holdCtrlToSelectMultiple() + "'></i>");
		htmlSitesAllowed.addStyleName("span3");

		final ListBox lBoxSitesAllowed = new ListBox(true);
		lBoxSitesAllowed.setStyleName("span9");
		// on parcourt la liste de tous les SiteProxy et de tous les user.group.siteAllowed pour les ajouter à la listBox (on marquera "(Groupe)" pour les sites déjà sélectionné par le groupe du user)
		for(int i=0; i<listSites.size(); i++){
			boolean isInGroup = false;
			for(SiteProxy siteGroup : user.getGroup().getSiteAllowed()){
				if(listSites.get(i).getName().equals(siteGroup.getName())){
					isInGroup = true;
					break;
				}
			}
			// l'ajoute à la ListBox de Site avec/sans le terme "(Groupe)"
			String inGroupOrDisabled = "";
			if(isInGroup && !listSites.get(i).isEnabled())
				inGroupOrDisabled += " (" + constants.group() + "-" + constants.disabled() + ")";	// (Group-Disabled)
			else if(isInGroup)
				inGroupOrDisabled += " (" + constants.group() + ")";								// (Group)
			else if(!listSites.get(i).isEnabled())
				inGroupOrDisabled += " (" + constants.disabled() + ")";							// (Disabled)
			lBoxSitesAllowed.addItem(listSites.get(i).getName() + inGroupOrDisabled, listSites.get(i).getName());

			// Si fais partie des sitesAllowed par le user, alors on le sélectionne
			if(user.getSiteAllowed().contains(listSites.get(i)))
				lBoxSitesAllowed.setItemSelected(i, true);
		}
		if(!UserManager.getInstance().isEditableUser())
			lBoxSitesAllowed.setEnabled(false);
		widgets.add(lBoxSitesAllowed);
		FlowPanel wraperSitesAllowed = new FlowPanel();
		wraperSitesAllowed.setStyleName("row-fluid");
		wraperSitesAllowed.add(htmlSitesAllowed);
		wraperSitesAllowed.add(lBoxSitesAllowed);


		// Mot de passe
		Label labelPassword = new Label(constants.password());
		labelPassword.addStyleName("span3");

		Anchor anchorPassword = new Anchor(constants.changePassword());
		anchorPassword.setHref("#modalPassword");
		if(UserManager.getInstance().isEditableUser())
			anchorPassword.getElement().setAttribute("data-toggle", "modal");
		else
			anchorPassword.setEnabled(false);
		anchorPassword.setStyleName("span9 btn no-margin-left");
		anchorPassword.addClickHandler(new ClickHandler() {
			@Override
			public void onClick(ClickEvent event) {
				loginSelected = user.getLogin();
				ptBox_changePassword1.setText("");
				ptBox_changePassword2.setText("");
			}
		});
		FlowPanel wraperPassword = new FlowPanel();
		wraperPassword.setStyleName("row-fluid");
		wraperPassword.add(labelPassword);
		wraperPassword.add(anchorPassword);

		// Etat
		Label labelState = new Label(constants.changeState());
		labelState.addStyleName("span3");

		final Button buttonShow = new Button(constants.enabled());
		buttonShow.setStyleName("btn");
		Button buttonHide = new Button(constants.disabled());
		buttonHide.setStyleName("btn");
		if(user.isEnabled()){
			buttonShow.addStyleName("active");
		}else{
			buttonHide.addStyleName("active");
		}
		if(!UserManager.getInstance().getLogin().equals(user.getLogin())){
			// si ce n'est pas l'utilisateur courant ...
			buttonShow.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					buttonUpdate.setVisible(true);
				}
			});
			buttonHide.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					buttonUpdate.setVisible(true);
				}
			});
		}else{
			buttonShow.setEnabled(false);
			buttonHide.setEnabled(false);
		}
		FlowPanel wraperVisibility = new FlowPanel();
		wraperVisibility.setStyleName("row-fluid padding-top-10");

		FlowPanel fPanelWrapperButtonVisibility = new FlowPanel();
		fPanelWrapperButtonVisibility.addStyleName("btn-group");
		if(UserManager.getInstance().isEditableUser())
			fPanelWrapperButtonVisibility.getElement().setAttribute("data-toggle","buttons-radio");
		fPanelWrapperButtonVisibility.add(buttonShow);
		fPanelWrapperButtonVisibility.add(buttonHide);

		wraperVisibility.add(labelState);
		wraperVisibility.add(fPanelWrapperButtonVisibility);

		// ajout des handlers
		for(Widget w : widgets){
			w.sinkEvents(Event.ONCHANGE);
			w.addHandler(new ChangeHandler() {
				@Override
				public void onChange(ChangeEvent event) {
					buttonUpdate.setVisible(true);
				}
			}, ChangeEvent.getType());
		}
		tBoxFirstName.addKeyUpHandler(new KeyUpHandler() {
			@Override
			public void onKeyUp(KeyUpEvent event) {
				buttonUpdate.setVisible(true);
			}
		});
		tBoxLastName.addKeyUpHandler(new KeyUpHandler() {
			@Override
			public void onKeyUp(KeyUpEvent event) {
				buttonUpdate.setVisible(true);
			}
		});
		tBoxMail.addKeyUpHandler(new KeyUpHandler() {
			@Override
			public void onKeyUp(KeyUpEvent event) {
				buttonUpdate.setVisible(true);
			}
		});
		
		
		// Mettre à jour
		FlowPanel wraperUpdate = null;
		if(UserManager.getInstance().isEditableUser()){
			buttonUpdate.setVisible(false);
			buttonUpdate.setStyleName("btn btn-info");
			buttonUpdate.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					String login = user.getLogin();
					String firstName = tBoxFirstName.getText();
					String lastName = tBoxLastName.getText();
					String mail = tBoxMail.getText();
					String groupSelected = lBoxGroup.getItemText(lBoxGroup.getSelectedIndex());
					Set<SiteProxy> siteAllowed = new HashSet<SiteProxy>(0);
					for(int i=0; i<lBoxSitesAllowed.getItemCount(); i++){
						if(lBoxSitesAllowed.isItemSelected(i)){
							SiteProxy siteSelected = getSiteProxy(lBoxSitesAllowed.getValue(i));
							if (siteSelected != null)
								siteAllowed.add(siteSelected);
						}
					}
					// si on n'a pas changé le mot de passe, "password" sera à null
					boolean enabled = (buttonShow.getStyleName().contains("active")) ? true : false;
					buttonUpdate.setVisible(false);
					presenter.updateUser(login, firstName, lastName, mail, siteAllowed, groupSelected, enabled);
				}
			});
			wraperUpdate = new FlowPanel();
			wraperUpdate.setStyleName("row-fluid padding-10");
			wraperUpdate.add(buttonUpdate);
		}

		fPanelContent.add(wraperFirstName);
		fPanelContent.add(wraperLastName);
		fPanelContent.add(wraperMail);
		fPanelContent.add(wraperPassword);
		fPanelContent.add(wraperGroup);
		fPanelContent.add(wraperSitesAllowed);
		fPanelContent.add(wraperPassword);
		fPanelContent.add(wraperVisibility);
		fPanelUser.add(fPanelContent);
		if(UserManager.getInstance().isEditableUser())
			fPanelUser.add(wraperUpdate);


		FlowPanel fPanelWrapperUser = new FlowPanel();
		fPanelWrapperUser.addStyleName("row-fluid");
		fPanelWrapperUser.add(fPanelUser);

		fPanel_listUser.add(fPanelWrapperUser);
	}

	@Override
	public void setListGroup(List<GroupProxy> listGroup) {
		// Efface notre liste de Group
		this.listGroup.clear();
		// et la remplace par la nouvelle
		this.listGroup.addAll(listGroup);

		// Efface la liste déroulante
		lBox_groupUser.clear();
		// et la remplace par les nouveaux items
		for(int i=0; i<listGroup.size(); i++){
			lBox_groupUser.addItem(listGroup.get(i).getName());
		}
	}

	@Override
	public void setListSite(List<SiteProxy> listSites) {
		// Efface notre liste de Sites
		this.listSites.clear();
		// et la remplace par la nouvelle
		this.listSites.addAll(listSites);

		// Efface la liste déroulante du modal "Ajouter un nouveau profil"
		lBox_sitesAllowed.clear();
		// et la remplace par les nouveaux items
		for(int i=0; i<listSites.size(); i++){
			lBox_sitesAllowed.addItem(listSites.get(i).getName());
		}
	}

	@Override
	public void displayPagination(int currentPage, int nbPage) {
		// Page précédente
		if(currentPage == 1){
			// désactivé
			li_upFirst.addClassName("disabled");
			li_upPrevious.addClassName("disabled");
			li_downFirst.addClassName("disabled");
			li_downPrevious.addClassName("disabled");
		}else{
			// activé
			li_upFirst.removeClassName("disabled");
			li_upPrevious.removeClassName("disabled");
			li_downFirst.removeClassName("disabled");
			li_downPrevious.removeClassName("disabled");
		}

		// Page suivante
		if(currentPage >= nbPage){
			// désactivé
			li_upLast.addClassName("disabled");
			li_upNext.addClassName("disabled");
			li_downLast.addClassName("disabled");
			li_downNext.addClassName("disabled");
		}else{
			// activé
			li_upLast.removeClassName("disabled");
			li_upNext.removeClassName("disabled");
			li_downLast.removeClassName("disabled");
			li_downNext.removeClassName("disabled");
		}

		// on enlève les numéros de page désactivé
		li_upPageE.removeClassName("disabled");
		li_upPageD.removeClassName("disabled");
		li_upPageC.removeClassName("disabled");
		li_upPageB.removeClassName("disabled");
		li_upPageA.removeClassName("disabled");
		li_downPageE.removeClassName("disabled");
		li_downPageD.removeClassName("disabled");
		li_downPageC.removeClassName("disabled");
		li_downPageB.removeClassName("disabled");
		li_downPageA.removeClassName("disabled");

		// on enlève les pages active
		li_upPageA.removeClassName("active");
		li_upPageB.removeClassName("active");
		li_upPageC.removeClassName("active");
		li_upPageD.removeClassName("active");
		li_upPageE.removeClassName("active");
		li_downPageA.removeClassName("active");
		li_downPageB.removeClassName("active");
		li_downPageC.removeClassName("active");
		li_downPageD.removeClassName("active");
		li_downPageE.removeClassName("active");

		if ( nbPage > 5){
			// si 6 pages ou plus
			if(currentPage<3){
				// CAS : 2 premières pages (exemple : pour la page 2/10, on affichera {1 2 3 4 5})
				// afficher curentPage-2, curentPage-1, curentPage, curentPage+1, curentPage+2
				anchor_upPageA.setText("1");
				anchor_upPageB.setText("2");
				anchor_upPageC.setText("3");
				anchor_upPageD.setText("4");
				anchor_upPageE.setText("5");
				anchor_downPageA.setText("1");
				anchor_downPageB.setText("2");
				anchor_downPageC.setText("3");
				anchor_downPageD.setText("4");
				anchor_downPageE.setText("5");
			}else if(nbPage-2 > currentPage){
				// CAS : ni les premières pages, ni les dernières (exemple : pour la page 4/10, on affichera {2 3 4 5 6})
				// afficher curentPage-2, curentPage-1, curentPage, curentPage+1, curentPage+2
				anchor_upPageE.setText(currentPage+2 + "");
				anchor_upPageD.setText(currentPage+1 + "");
				anchor_upPageC.setText(currentPage + "");
				anchor_upPageB.setText(currentPage-1 + "");
				anchor_upPageA.setText(currentPage-2 + "");
				anchor_downPageE.setText(currentPage+2 + "");
				anchor_downPageD.setText(currentPage+1 + "");
				anchor_downPageC.setText(currentPage + "");
				anchor_downPageB.setText(currentPage-1 + "");
				anchor_downPageA.setText(currentPage-2 + "");
			}else{
				// CAS : 3 dernières pages
				// afficher nbPage-4, nbPage-3, nbPage-2, nbPage1-1, nbPage
				anchor_upPageA.setText(nbPage-4 + "");
				anchor_upPageB.setText(nbPage-3 + "");
				anchor_upPageC.setText(nbPage-2 + "");
				anchor_upPageD.setText(nbPage-1 + "");
				anchor_upPageE.setText(nbPage   + "");
				anchor_downPageA.setText(nbPage-4 + "");
				anchor_downPageB.setText(nbPage-3 + "");
				anchor_downPageC.setText(nbPage-2 + "");
				anchor_downPageD.setText(nbPage-1 + "");
				anchor_downPageE.setText(nbPage   + "");
			}
		}else{
			// CAS : si moins de 5 pages
			anchor_upPageA.setText("1");
			anchor_upPageB.setText("2");
			anchor_upPageC.setText("3");
			anchor_upPageD.setText("4");
			anchor_upPageE.setText("5");
			anchor_downPageA.setText("1");
			anchor_downPageB.setText("2");
			anchor_downPageC.setText("3");
			anchor_downPageD.setText("4");
			anchor_downPageE.setText("5");
			// ajoute "disabled" pour désactivé la page dont on n'a pas accès
			switch (nbPage) {
			case 1:
				li_upPageB.addClassName("disabled");
				li_downPageB.addClassName("disabled");
			case 2:
				li_upPageC.addClassName("disabled");
				li_downPageC.addClassName("disabled");
			case 3:
				li_upPageD.addClassName("disabled");
				li_downPageD.addClassName("disabled");
			case 4:
				li_upPageE.addClassName("disabled");
				li_downPageE.addClassName("disabled");
				break;
			default:
				li_upPageA.addClassName("disabled");
				li_downPageA.addClassName("disabled");
				li_upPageB.addClassName("disabled");
				li_downPageB.addClassName("disabled");
				li_upPageC.addClassName("disabled");
				li_downPageC.addClassName("disabled");
				li_upPageD.addClassName("disabled");
				li_downPageD.addClassName("disabled");
				li_upPageE.addClassName("disabled");
				li_downPageE.addClassName("disabled");
				break;
			}

		}

		// on passe à "active" la bonne page
		if(anchor_upPageA.getText().equals(currentPage+"")){
			li_upPageA.addClassName("active");
			li_downPageA.addClassName("active");
		}else if(anchor_upPageB.getText().equals(currentPage+"")){
			li_upPageB.addClassName("active");
			li_downPageB.addClassName("active");
		}else if(anchor_upPageC.getText().equals(currentPage+"")){
			li_upPageC.addClassName("active");
			li_downPageC.addClassName("active");
		}else if(anchor_upPageD.getText().equals(currentPage+"")){
			li_upPageD.addClassName("active");
			li_downPageD.addClassName("active");
		}else if(anchor_upPageE.getText().equals(currentPage+"")){
			li_upPageE.addClassName("active");
			li_downPageE.addClassName("active");
		}

		for(HandlerRegistration h : handlerPagination){
			h.removeHandler();
		}

		if (! li_upPageA.getClassName().contains("disabled")){
			handlerPagination.add(anchor_upPageA.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					presenter.getUserPage(Integer.parseInt(anchor_upPageA.getText()));
				}
			}));
			handlerPagination.add(anchor_downPageA.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					presenter.getUserPage(Integer.parseInt(anchor_downPageA.getText()));
				}
			}));
		}
		if (! li_upPageB.getClassName().contains("disabled")){
			handlerPagination.add(anchor_upPageB.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					presenter.getUserPage(Integer.parseInt(anchor_upPageB.getText()));
				}
			}));
			handlerPagination.add(anchor_downPageB.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					presenter.getUserPage(Integer.parseInt(anchor_downPageB.getText()));
				}
			}));
		}
		if (! li_upPageC.getClassName().contains("disabled")){
			handlerPagination.add(anchor_upPageC.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					presenter.getUserPage(Integer.parseInt(anchor_upPageC.getText()));
				}
			}));
			handlerPagination.add(anchor_downPageC.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					presenter.getUserPage(Integer.parseInt(anchor_downPageC.getText()));
				}
			}));
		}
		if (! li_upPageD.getClassName().contains("disabled")){
			handlerPagination.add(anchor_upPageD.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					presenter.getUserPage(Integer.parseInt(anchor_upPageD.getText()));
				}
			}));
			handlerPagination.add(anchor_downPageD.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					presenter.getUserPage(Integer.parseInt(anchor_downPageD.getText()));
				}
			}));
		}
		if (! li_upPageE.getClassName().contains("disabled")){
			handlerPagination.add(anchor_upPageE.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					presenter.getUserPage(Integer.parseInt(anchor_upPageE.getText()));
				}
			}));
			handlerPagination.add(anchor_downPageE.addClickHandler(new ClickHandler() {
				@Override
				public void onClick(ClickEvent event) {
					presenter.getUserPage(Integer.parseInt(anchor_downPageE.getText()));
				}
			}));
		}


		/* OLD
		if(nbPage<=1){
			label_numPage1.setVisible(false);
			label_numPage2.setVisible(false);
			button_previous1.setVisible(false);
			button_previous2.setVisible(false);
			button_next1.setVisible(false);
			button_next2.setVisible(false);
		}else{
			label_numPage1.setVisible(true);
			label_numPage2.setVisible(true);
			button_previous1.setVisible(true);
			button_previous2.setVisible(true);
			button_next1.setVisible(true);
			button_next2.setVisible(true);

			label_numPage1.setText(currentPage + "/" + nbPage);
			label_numPage2.setText(currentPage + "/" + nbPage);

			button_previous1.removeStyleName("disabled");
			button_previous2.removeStyleName("disabled");

			button_next1.removeStyleName("disabled");
			button_next2.removeStyleName("disabled");
			if(currentPage<=1){
				button_previous1.addStyleName("disabled");
				button_previous2.addStyleName("disabled");
			}
			if(currentPage>=nbPage){
				button_next1.addStyleName("disabled");
				button_next2.addStyleName("disabled");
			}
		}*/
	}

	@Override
	public void clearPanelUser() {
		fPanel_listUser.clear();
	}

	@Override
	public void loginAvailable(boolean available){
		fPanel_loginDisponibility.clear();
		if(available){
			fPanel_loginDisponibility.setStyleName("label label-success pull-right");
			fPanel_loginDisponibility.add(new HTML(constants.loginAvailable()));
		}else{
			fPanel_loginDisponibility.setStyleName("label label-important pull-right");
			fPanel_loginDisponibility.add(new HTML(constants.loginUnavailable()));
		}
		fPanel_loginDisponibility.setVisible(true);
	}


	/* Fonction */

	/**
	 * Retourne le SiteProxy à partir de son nom. Null si le SiteProxy n'est pas trouvé.
	 * @param siteName
	 * @return
	 */
	private SiteProxy getSiteProxy(String siteName){
		for(SiteProxy s : listSites){
			if(s.getName().equals(siteName)){
				return s;
			}

		}
		return null;
	}


	/* Handler */

	/**
	 * Pagination up : Page précédente.
	 * @param e
	 */
	@UiHandler("anchor_upPrevious")
	public void onClickUpPreviousPage(ClickEvent e){
		if(UserManager.getInstance().isViewableUser() && !li_upPrevious.getClassName().contains("disabled"))
			presenter.previousPage();
	}
	/**
	 * Pagination up : Page suivante.
	 * @param e
	 */
	@UiHandler("anchor_upNext")
	public void onClickUpNextPage(ClickEvent e){
		if(UserManager.getInstance().isViewableUser() && !li_upNext.getClassName().contains("disabled"))
			presenter.nextPage();
	}
	/**
	 * Pagination up : Première page.
	 * @param e
	 */
	@UiHandler("anchor_upFirst")
	public void onClickUpFirstPage(ClickEvent e){
		if(UserManager.getInstance().isViewableUser() && !li_upFirst.getClassName().contains("disabled"))
			presenter.getUserPage(1);
	}
	/**
	 * Pagination up : Dernière page.
	 * @param e
	 */
	@UiHandler("anchor_upLast")
	public void onClickUpLastPage(ClickEvent e){
		if(UserManager.getInstance().isViewableUser() && !li_upLast.getClassName().contains("disabled"))
			presenter.lastPage();
	}
	
	/**
	 * Pagination down : Page précédente.
	 * @param e
	 */
	@UiHandler("anchor_upPrevious")
	public void onClickDownPreviousPage(ClickEvent e){
		if(UserManager.getInstance().isViewableUser() && !li_downPrevious.getClassName().contains("disabled"))
			presenter.previousPage();
	}
	/**
	 * Pagination down : Page suivante.
	 * @param e
	 */
	@UiHandler("anchor_upNext")
	public void onClickDownNextPage(ClickEvent e){
		if(UserManager.getInstance().isViewableUser() && !li_downNext.getClassName().contains("disabled"))
			presenter.nextPage();
	}
	/**
	 * Pagination down : Première page.
	 * @param e
	 */
	@UiHandler("anchor_upFirst")
	public void onClickDownFirstPage(ClickEvent e){
		if(UserManager.getInstance().isViewableUser() && !li_downFirst.getClassName().contains("disabled"))
			presenter.getUserPage(1);
	}
	/**
	 * Pagination down : Dernière page.
	 * @param e
	 */
	@UiHandler("anchor_upLast")
	public void onClickDownLastPage(ClickEvent e){
		if(UserManager.getInstance().isViewableUser() && !li_downLast.getClassName().contains("disabled"))
			presenter.lastPage();
	}
	

	@UiHandler("anchor_addUser")
	public void onClickAddUser(ClickEvent e){
		// Réinitialise les valeurs par défaut
		tBox_loginUser.setText("");
		tBox_firstNameUser.setText("");
		tBox_lastNameUser.setText("");
		tBox_mailUser.setText("");
		ptBox_passwordUser1.setText("");
		ptBox_passwordUser2.setText("");
		button_enabled.setStyleName("btn active");
		button_disabled.setStyleName("btn");
		fPanel_loginDisponibility.setVisible(false);
	}

	@UiHandler("button_validAddUser")
	public void onClickValid(ClickEvent e){
		if(UserManager.getInstance().isCreatableUser()){
			// récupération des champs
			String loginUser = tBox_loginUser.getText();
			String firstNameUser = tBox_firstNameUser.getText();
			String lastNameUser = tBox_lastNameUser.getText();
			String mailUser = tBox_mailUser.getText();
			String passwordUser1 = ptBox_passwordUser1.getText();
			String passwordUser2 = ptBox_passwordUser2.getText();
			String roleUser = lBox_groupUser.getValue(lBox_groupUser.getSelectedIndex());
			boolean enabledUser = (button_enabled.getStyleName().contains("active")) ? true : false;
			Set<SiteProxy> sitesAllowed = new HashSet<SiteProxy>(0);
			for(int i=0; i<lBox_sitesAllowed.getItemCount(); i++){
				if(lBox_sitesAllowed.isItemSelected(i))
					// si sélectionné dans la ListBox, alors ajouté dans le HashSet
					sitesAllowed.add(getSiteProxy(lBox_sitesAllowed.getItemText(i)));
			}

			// vérification de leurs complétude
			if(loginUser.equals("")){
				Window.alert(constants.pleaseEnterLogin());
			}else if(firstNameUser.equals("")){
				Window.alert(constants.pleaseEnterFirstName());
			}else if(lastNameUser.equals("")){
				Window.alert(constants.pleaseEnterLastName());
			}else if(passwordUser1.equals("")){
				Window.alert(constants.pleaseEnterPassword());
			}else if(passwordUser2.equals("")){
				Window.alert(constants.pleaseEnterRepeatPassword());
			}else if(! ptBox_passwordUser1.getText().equals(ptBox_passwordUser2.getText())){
				Window.alert(constants.passwordDifferent());
			}else{
				presenter.createUser(loginUser, firstNameUser, lastNameUser, mailUser, roleUser , passwordUser1, sitesAllowed, enabledUser);
			}
		}
	}

	@UiHandler("button_validNewPassword")
	public void onClickValidNewPassword(ClickEvent e){
		if( ! ptBox_changePassword1.getText().equals(ptBox_changePassword2.getText())){
			// 2 passwords différents
			Window.alert(constants.passwordDifferent());
		}else if(ptBox_changePassword1.getText().equals("")){
			// password vide
			Window.alert(constants.pleaseEnterPassword());
		}else{
			// 2 passwords identiques : on update !
			presenter.updatePassword(loginSelected, ptBox_changePassword1.getText());
			// et on efface les champs de saisie
			ptBox_changePassword1.setText("");
			ptBox_changePassword2.setText("");
		}
		// dans btous les cas, on efface la variable loginSelected (instancié au moment ou on a clické sur le bouton "changer password")
		loginSelected = null;
	}


}
