package fr.janua.client.view.interfaces;

import java.util.List;
import java.util.Set;

import com.google.gwt.user.client.ui.IsWidget;
import com.mvp4g.client.view.LazyView;

import fr.janua.client.proxy.MessageProxy;
import fr.janua.client.proxy.SiteProxy;
import fr.janua.client.proxy.TypeProxy;
import fr.janua.client.proxy.UserProxy;

public interface IPageMainCouranteView extends IsWidget , LazyView {
	public interface IPageMainCourantePresenter{

		/**
		 * Crée un Message.
		 * @param title
		 * @param description
		 * @param site
		 * @param type
		 */
		void createMessage(String title, String description, SiteProxy site, Set<TypeProxy> type);

		/**
		 * Envoie une requête pour récupérer le nombre de messages ouverts.
		 */
		void getNbOpenMessage();
		
		/**
		 * Envoie une requête pour récupérer le nombre de messages ouverts.
		 */
		void getNbClosedMessage();
		
		/**
		 * Met à jour le message passé en paramètre.
		 * @param message
		 * @param comment
		 */
		void closeMessage(MessageProxy message, String comment);


		/**
		 * Demande au presenter d'aller à la page précédente des Messages ouverts.
		 */
		void previousPageOpenMessage();
		
		/**
		 * Demande au presenter d'aller à la page suivante des Messages ouverts.
		 */
		void nextPageOpenMessage();

		/**
		 * Demande au presenter d'aller à la page précédente  des Messages fermés.
		 */
		void previousPageClosedMessage();
		/**
		 * Demande au presenter d'aller à la page suivante des Messages fermés.
		 */
		void nextPageClosedMessage();

		/**
		 * Ajoute un commentaire au message
		 * @param messageForComment
		 * @param comment
		 */
		void addComment(MessageProxy messageForComment, String comment);


		void search(String wordSearch);

		/**
		 * Ajoute un type
		 * @param message
		 * @param itemText
		 */
		void addType(MessageProxy message, String itemText);

		/**
		 * Récupère les info sur le profil pour les afficher.
		 * @param nameCreator
		 */
		void getProfile(String nameCreator);

		/**
		 * Récupère la liste de tous les Login de Users.
		 */
		void getAllLoginUser();

		/**
		 * Met à jour le champ "lastActivity" du User pour qu'il soit postérieur aux dates des messages. Ce qui permet de ne plus afficher le "NEW" sur les messages.
		 */
		void updateLastActivity();
		
		/**
		 * Récupère la page <code>numPage</code> des messages ouverts. 
		 * @param numPage
		 */
		void getOpenMessagePage(int numPage);

		/**
		 * Récupère la page <code>numPage</code> des messages fermés.
		 * @param numPage
		 */
		void getClosedMessagePage(int numPage);

		/**
		 * Récupère la dernière page des messages ouverts.
		 */
		void lastPageOpenMessage();

		/**
		 * Récupère la dernière page des messages fermés.
		 */
		void lastPageClosedMessage();

		

		
		
	}

	/**
	 * Fonction appelée à chaque fois que l'on va charger la page.
	 * @param name
	 */
	void loadPage();

	/**
	 * Remplie la liste déroulante des nom de type correspondant.
	 * @param listType
	 */
//	void displayListBoxType(List<OtherTypeProxy> listType);

	/**
	 * Remplie la liste déroulante des nom de sites correspondant.
	 * @param listSite
	 */
//	void displayListBoxSite(List<OtherSiteProxy> listSite);

	/**
	 * Affiche un Message dans les messages ouverts.
	 * @param message
	 */
	void displayOneMessageInOpenMessage(MessageProxy message);

	/**
	 * Affiche un séparateur pour les Messages ouverts (une ligne horizontale de 1px).
	 */
	void displaySeparatorInOpenMessages();

	/**
	 * Affiche un Message dans les messages fermés.
	 * @param message
	 */
	void displayOneMessageInClosedMessage(MessageProxy message);

	/**
	 * Affiche un séparateur pour les Messages fermés (une ligne horizontale de 1px).
	 */
	void displaySeparatorInClosedMessages();

	/**
	 * Affiche un Message dans les résultats de la recherche.
	 * @param message
	 */
	void displayOneMessageInMessagesFound(MessageProxy m);
	
	/**
	 * Affiche un séparateur pour les Messages fermés (une ligne horizontale de 1px).
	 */
	void displaySeparatorInMessagesFound();

	/**
	 * Réactive et nettoie les champs de création de message, réaffiche le bouton "Aperçu", efface le Panel d'aperçu, et masque les boutons "Modifer" et "Valider".
	 * Appelé lorsque le message a bien été crée.
	 */
	void resetFieldCreateMessage();

	/**
	 * Efface tous les messages ouverts.
	 */
	void clearOpenMessage();
	/**
	 * Efface tous les messages fermés.
	 */
	void clearClosedMessage();
	/**
	 * Efface tous les messages de la recherche.
	 */
	void clearMessagesFound();
	
	/**
	 * Affiche la pagination pour les messages ouverts.
	 * @param currentPageOpenMessage
	 * @param nbPage
	 */
	void displayPaginationOpenMessage(int currentPageOpenMessage, int nbPage);

	/**
	 * Affiche la pagination pour les messages fermés.
	 * @param currentPageClosedMessage
	 * @param nbPage
	 */
	void displayPaginationClosedMessage(int currentPageClosedMessage, int nbPage);


	void setNbOpenMessage(Integer nbResults);
	void setNbClosedMessage(Integer nbResults);

	void setListLogin(List<String> listLogins);
	
	/**
	 * Se met sur l'onglet "Messages ouverts"
	 */
	void goToTabOpenMessage();

	/**
	 * Affiche le profil récupéré dans le modal.
	 * @param user
	 */
	void displayProfileInModal(UserProxy user);

	/**
	 * Actualise les résultats de la recherche.
	 */
	void refreshSearch();

	/**
	 * Affiche le message "Aucun résultat.".
	 */
	void noResultsFound();

	


}
