package fr.janua.server;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import fr.janua.server.dao.DAOFactory;
import fr.janua.server.dao.object.LogDAO;
import fr.janua.server.dao.object.MessageDAO;
import fr.janua.server.entity.Comment;
import fr.janua.server.entity.Message;
import fr.janua.server.entity.Type;
import fr.janua.shared.Log;

@SuppressWarnings("serial")
public class DownloadCSV extends HttpServlet {

	private static final char SEPARATOR = ';';
	
	@Override
	protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException { 
		super.doGet(req, resp);
	}

	@Override
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		/*
		 * Exemple de requête :
		 * 	/csv?query=SELECT{......}&type=message
		 *  /csv?query=SELECT{......}&type=log
		 */
		
		String query = 		 request.getParameter("query");
		String type = 		 request.getParameter("type");	// "message" OU "log"
		
		if (query == null || type == null){
			response.getWriter().write("Malformed request !");
			return;
		}
		String csvFileName = "";
		File csvFile = null;
		if("message".equals(type)){
			MessageDAO messageDAO = DAOFactory.getMessageDAO();
			List<Message> listMessage = messageDAO.search(query);
	
			csvFileName = "Jaguards-closed-message";
			for(Message m : listMessage)
				csvFileName += "-" + m.getId();
			csvFileName += ".csv";
	
			String contextPath = PdfManager.getInstance().getFullPathCSV();	//getServletContext().getRealPath(File.separator + "pdf" + File.separator);
			csvFile = new File(contextPath + csvFileName);
			
			generateCsvFileForMessage(contextPath + csvFileName, listMessage);

			response.setContentType("application/csv");
			response.addHeader("Content-Disposition", "attachment; filename=" + csvFileName);
			response.setContentLength((int) csvFile.length());
			
			@SuppressWarnings("resource")
			FileInputStream fileInputStream = new FileInputStream(csvFile);
			OutputStream responseOutputStream = response.getOutputStream();
			int bytes;
			while ((bytes = fileInputStream.read()) != -1) {
				responseOutputStream.write(bytes);
			}
		}else if("log".equals(type)){
			LogDAO logsDAO = DAOFactory.getLogDAO();
			List<Log> listLogs = logsDAO.findAllLog();
	
			csvFileName = "Jaguards-logs-" + longToDate(System.currentTimeMillis()) + "(" + listLogs.size() + ").csv";
	
			String contextPath = PdfManager.getInstance().getFullPathCSV();
			csvFile = new File(contextPath + csvFileName);
			
			generateCsvFileForLog(contextPath + csvFileName, listLogs);
			
			response.setContentType("application/csv");
			response.addHeader("Content-Disposition", "attachment; filename=" + csvFileName);
			response.setContentLength((int) csvFile.length());
			
			@SuppressWarnings("resource")
			FileInputStream fileInputStream = new FileInputStream(csvFile);
			OutputStream responseOutputStream = response.getOutputStream();
			int bytes;
			while ((bytes = fileInputStream.read()) != -1) {
				responseOutputStream.write(bytes);
			}
		}else{
			response.getWriter().write("Type not found !");
			return;
		}
		
		
	}

	/**
	 * Génère le csv pour les messages.
	 * @param pathFileName
	 * @param listMessage
	 */
	private void generateCsvFileForMessage(String pathFileName, List<Message> listMessage) {
		try {
			FileWriter writer = new FileWriter(pathFileName);

			// Ligne 1 : les titres
			writer.append("ID");
			writer.append(SEPARATOR);
			writer.append("Title");
			writer.append(SEPARATOR);
			writer.append("Status");
			writer.append(SEPARATOR);
			writer.append("Date Creation");
			writer.append(SEPARATOR);
			writer.append("Date Closure");
			writer.append(SEPARATOR);
			writer.append("Site");
			writer.append(SEPARATOR);
			writer.append("Type");
			writer.append(SEPARATOR);
			writer.append("Description");
			writer.append(SEPARATOR);
			writer.append("Comment closure");
			writer.append(SEPARATOR);
			writer.append("Comments");
			writer.append('\n');

			// Ligne n
			for(Message m : listMessage){
				writer.append(m.getId() + "");
				writer.append(SEPARATOR);
				writer.append(m.getTitle());
				writer.append(SEPARATOR);
				writer.append(m.getStatus() + "");
				writer.append(SEPARATOR);
				writer.append(longToDate(m.getCreationDate()));
				writer.append(SEPARATOR);
				writer.append(longToDate(m.getClosureDate()));
				writer.append(SEPARATOR);
				writer.append(m.getSite().getName());
				writer.append(SEPARATOR);
				String types = "";
				for(Type t : m.getTypes()){
					types += t.getName() + "-";
				}
				types.substring(0, types.length()-2);
				writer.append(types);
				writer.append(SEPARATOR);
				writer.append(m.getDescription());
				writer.append(SEPARATOR);
				writer.append(m.getCommentClosure());
				writer.append(SEPARATOR);
				for(Comment c : m.getComments()){
					if(c.getType() == null){
						writer.append(c.getAuthor() + " : " + c.getDescription());
						writer.append(SEPARATOR);
					}else{
						writer.append(c.getAuthor() + " add Type : {" + c.getType().getName() + "}");
						writer.append(SEPARATOR);
					}
				}
				writer.append('\n');
			}
			//generate whatever data you want

			writer.flush();
			writer.close();
		}catch(IOException e) {
			e.printStackTrace();
		} 
	}

	/**
	 * Génère le csv pour les logs.
	 * @param pathFileName
	 * @param listLogs
	 */
	private void generateCsvFileForLog(String pathFileName, List<Log> listLogs) {
		try {
			FileWriter writer = new FileWriter(pathFileName);

			// Ligne 1 : les titres
			writer.append("ID");
			writer.append(SEPARATOR);
			writer.append("Date");
			writer.append(SEPARATOR);
			writer.append("Action");
			writer.append(SEPARATOR);
			writer.append("Description");
			writer.append(SEPARATOR);
			writer.append("Author");
			writer.append('\n');

			// Ligne n
			for(Log m : listLogs){
				writer.append(m.getId() + "");
				writer.append(SEPARATOR);
				writer.append(longToDate(m.getDate()));
				writer.append(SEPARATOR);
				writer.append(m.getAction() + "");
				writer.append(SEPARATOR);
				writer.append(m.getDetail() + "");
				writer.append(SEPARATOR);
				writer.append(m.getLoginAuthor());
				writer.append('\n');
			}

			writer.flush();
			writer.close();
		}catch(IOException e) {
			e.printStackTrace();
		} 
	}


	/**
	 * Retourne la date sous forme "dd/mm/yyyy - hh:mm:ss"
	 * @param l
	 * @return
	 */
	private String longToDate(long time){
		String[] tabMonth = {"error Date", "janvier", "février", "mars", "avril", "mai", "juin", "juillet", "août", "septembre", "octobre", "novembre", "décembre"};
		String res = "";
		Date date = new Date(time);
		// 30 ...
		DateFormat fmt = new SimpleDateFormat("dd");
		res += fmt.format(date) + " ";
		// ... avril ...
		fmt = new SimpleDateFormat("MM");
		res += tabMonth[Integer.parseInt(fmt.format(date))] + " ";
		// ... 2013 à 12h22
		fmt = new SimpleDateFormat("yyyy - HH:mm:ss");
		res += fmt.format(date);
		return res;
	}
	
}
