package fr.janua.server;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.OutputStream;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import fr.janua.server.dao.DAOFactory;
import fr.janua.server.dao.object.MessageDAO;
import fr.janua.server.dao.object.UserDAO;
import fr.janua.server.entity.Message;
import fr.janua.server.entity.Site;
import fr.janua.server.entity.Type;
import fr.janua.server.entity.User;

@SuppressWarnings("serial")
public class DownloadPDF extends HttpServlet {

	@Override
	protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
		super.doGet(req, resp);
	}

	@Override
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		//response.getWriter().write("Hello "+ name +", Welcome in My Servlet");
		// Map params = request.getParameterMap();
		// Iterator i = params.keySet().iterator();
		// System.out.println("*****************************************************");
		// while ( i.hasNext() ){
		// String key = (String) i.next();
		// String value = ((String[]) params.get( key ))[ 0 ];
		// System.out.println(key + " - " + value);
		// }
		// System.out.println("*****************************************************");
		
		String login = request.getParameter("login");
		String password = request.getParameter("password");
		int idMessage = Integer.parseInt(request.getParameter("idMessage"));
		String pdfFileName = request.getParameter("namePDF");

		if(checkRights(login, password, idMessage)){
			String contextPath = PdfManager.getInstance().getFullPathPDF();	//getServletContext().getRealPath(File.separator + "pdf" + File.separator);
			File pdfFile = new File(contextPath + pdfFileName);

			// Si le fichier n'existe pas, on génère le pdf 
			if(!pdfFile.exists()){
				System.out.println("Le fichier \"" + contextPath + pdfFileName + "\" n'existe pas, on le regénère !");

				MessageDAO messageDAO = DAOFactory.getMessageDAO();
				Message m = messageDAO.findById(idMessage);
				
				PdfManager.getInstance().createPdf(m);
			}
			response.setContentType("application/pdf");
			response.addHeader("Content-Disposition", "attachment; filename=" + pdfFileName);
			response.setContentLength((int) pdfFile.length());

			@SuppressWarnings("resource")
			FileInputStream fileInputStream = new FileInputStream(pdfFile);
			OutputStream responseOutputStream = response.getOutputStream();
			int bytes;
			while ((bytes = fileInputStream.read()) != -1) {
				responseOutputStream.write(bytes);
			}
		}

	}

	/**
	 * Vérifie les droits d'accès au fichier.
	 * @param login
	 * @param encryptedPassword
	 * @param idMessage
	 * @return
	 */
	private boolean checkRights(String login, String encryptedPassword, int idMessage){
		// Check si le mot de passe et le login concordent
		UserDAO userDao = DAOFactory.getUserDAO();
		User user = userDao.checkWithEncryptedPaswword(login, encryptedPassword);
		if(user == null) return false;

		// Check le droit de voir le message
		if((user.getGroup().getCrudMessage())%2 == 0) return false;

		MessageDAO messageDao = DAOFactory.getMessageDAO();
		Message m = messageDao.findById(idMessage);

		// Check si les sites concordent
		boolean siteAllowed = false;
		for(Site s : user.getGroup().getSiteAllowed())
			if(s.getName().equals(m.getSite().getName()))
				siteAllowed = true;

		// si aucun site concorde, on ne va pas plus loin
		if (!siteAllowed)
			return false;

		// Check si les types concordent
		for(Type typeOfMessage : m.getTypes()){
			for(Type typeAllowed : user.getGroup().getTypeAllowed())
				if(typeAllowed.getName().equals(typeOfMessage.getName()))
					return true;
		}

		return false;
	}
}
