package fr.janua.server;

import java.io.File;
import java.io.IOException;
import java.util.Iterator;
import java.util.List;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileItemFactory;
import org.apache.commons.fileupload.FileUploadException;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.servlet.ServletFileUpload;

import fr.janua.server.dao.DAOFactory;
import fr.janua.server.dao.object.UserDAO;
import fr.janua.server.entity.User;

@SuppressWarnings("serial")
public class UploadPDFServlet extends HttpServlet {

	@Override
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		//		super.doGet(request, response);

		// factory for disk-based file items
		FileItemFactory factory = new DiskFileItemFactory();

		// file upload handler
		ServletFileUpload upload = new ServletFileUpload(factory);

		try{

			// parse la requête
			List<FileItem> items = upload.parseRequest(request);
			System.out.println("items.size : " + items.size());
			Iterator<FileItem> iterator = items.iterator();

			String login = "";
			String password = "";
			FileItem file = null;
			String pdfName = "";
			while(iterator.hasNext()){
				FileItem item = (FileItem) iterator.next();
				if(item.getFieldName().equals("file")){
					if(!item.isFormField()){
						file = item;
					}
				}else if(item.getFieldName().equals("login")){
					System.out.println("FieldName = login, value = " + item.getString() + ", name = " + item.getName());
					login = item.getString();
				}else if(item.getFieldName().equals("password")){
					System.out.println("FieldName = password, Value = " + item.getString() + ", name = " + item.getName());
					password = item.getString();
				}else if(item.getFieldName().equals("section")){
					System.out.println("FieldName = password, Value = " + item.getString() + ", name = " + item.getName());
					pdfName = item.getString() + pdfName;
				}else if(item.getFieldName().equals("language")){
					System.out.println("FieldName = password, Value = " + item.getString() + ", name = " + item.getName());
					pdfName = pdfName + item.getString();
				}
			}
			
			pdfName += ".pdf";

			// vérifie que le login/password sont bon
			boolean isGoodLogin = checkLogin(login, password);

			if (file != null && isGoodLogin){
				String root = getServletContext().getRealPath("/doc");
				System.out.println("Fichier uploadé dans : " + root);
				File path = new File(root);
				if(!path.exists())
					path.mkdir();

				File uploadedFile = new File(path + "/" + pdfName);
				// si le fichier existe déjà, on renomme l'ancien
				if(uploadedFile.exists())
					uploadedFile.renameTo(new File(path + "/(" + pdfName + ")_" + System.currentTimeMillis() + ".pdf"));
				System.out.println(uploadedFile.getAbsolutePath());
				file.write(uploadedFile);
				response.getWriter().write("Upload success !");
			}else if(!isGoodLogin){
				response.getWriter().write("No access rights !");
			}else{
				response.getWriter().write("Fail !");
			}

		}catch(FileUploadException e){
			e.printStackTrace();
			response.getWriter().write("FileUploadException !");
		}catch (Exception e) {
			e.printStackTrace();
			response.getWriter().write("Exception !");
		}

	}

	@Override
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		//response.getWriter().write("empty servlet !");


	}


	/**
	 * Retourne <code>true</code> si le login et le password correspondent avec un compte ayant les droits d'uploader un pdf
	 * @param login
	 * @param password
	 * @return
	 */
	private boolean checkLogin(String login, String password){

		UserDAO userDao = DAOFactory.getUserDAO();
		User u = userDao.checkWithEncryptedPaswword(login, password);
		if(u == null){
			return false;
		}else if(u.getGroup().getCrudUploadDoc() == 2 || u.getGroup().getCrudUploadDoc() == 3 || u.getGroup().getCrudUploadDoc() > 5){
			return true;
		}else{
			return false;
		}
	}
}
