package fr.janua.server.dao.object;

import java.util.List;

import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.Transaction;

import fr.janua.server.dao.DAO;
import fr.janua.server.entity.Group;
import fr.janua.server.entity.Site;
import fr.janua.server.entity.Type;
import fr.janua.server.utils.HibernateUtils;

public class GroupDAO extends DAO<Group>{

	public GroupDAO() {
		super();
	}

	@Override
	public boolean create(Group group) {
		Session session = HibernateUtils.getSessionFactory().openSession();
		Transaction tx = null;
		try {
			tx = session.beginTransaction();
			session.save(group);
			tx.commit();
		} catch (Exception e) {
			e.printStackTrace();
			if (tx != null && session.isOpen())
				tx.rollback();
			return false;
		}finally{
			session.close();
		}
		return true;
	}

	@Override
	public boolean delete(Group group) {
		// TODO : à coder
		return false;
	}

	@Override
	public boolean update(Group group) {
		Session session = HibernateUtils.getSessionFactory().openSession();
		Transaction tx = null;
		try {
			tx = session.beginTransaction();
			session.update(group);
			tx.commit();
		}catch (Exception e) {
			e.printStackTrace();
			if (tx != null && session.isOpen())
				tx.rollback();
			return false;
		}finally{
			session.close();
		}
		return true;
	}

	@Override
	public Group findById(int id) {
		System.err.println("Group n'a pas d'id. Utiliser la fonction finByName !");
		return null;
	}
	
	public Group findByName(String name) {
		Session session = HibernateUtils.getSessionFactory().openSession();
		Query query = session.createQuery("SELECT group FROM Group group WHERE group.name = :name");
		query.setParameter("name", name);
		Group group = (Group) query.uniqueResult();
		session.close();
		return group;
	}

	/**
	 * Récupère tous les Groupes de la base de donnée.
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public List<Group> findAllGroup() {
		Session session = HibernateUtils.getSessionFactory().openSession();
		Query query = session.createQuery("SELECT group FROM Group group");
		List<Group> list = query.list();
		session.close();
		
		return list;
	}

	/**
	 * Retourne <code>true</code> s'il existe au moins un groupe "Admin".
	 * Un groupe "Admin" est un groupe qui possède le droit de créer/éditer/voir les groupes. 
	 * @return
	 */
	public boolean isAdminExist(){
		Session session = HibernateUtils.getSessionFactory().openSession();
		Query query = session.createQuery(	"SELECT g FROM Group g WHERE g.crudGroup = :crudGroup");
		query.setParameter("crudGroup", 7);
		Group group = (Group) query.uniqueResult();
		
		session.close();
		if(group == null)
			return false;
		else
			return true;
	}

	/**
	 * Retourne n'importe quel groupe ayant les droits "Admin".
	 * "Droits admin" signifiant qu'il peut Créer/Editer/Voir les groupes.
	 * @return
	 */
	public Group getGroupAdmin(){
		Session session = HibernateUtils.getSessionFactory().openSession();
		Query query = session.createQuery("SELECT group FROM Group group WHERE group.crudGroup = :crudGroup");
		query.setParameter("crudGroup", 7);
		Group group = (Group) query.uniqueResult();
		session.close();
		return group;
	}
	
	public boolean addSite(String groupName, String siteName){
		Session session = HibernateUtils.getSessionFactory().openSession();
		// récupération du Site
		Query query = session.createQuery("SELECT site FROM Site site WHERE site.name = :name");
		query.setParameter("name", siteName);
		Site site = (Site) query.uniqueResult();
		if (site == null){
			System.err.println("Group#addSite() : nom du site inconnu !");
			session.close();
			return false;
		}
		
		// récupération du Group
		query = session.createQuery("SELECT group FROM Group group WHERE group.name = :name");
		query.setParameter("name", groupName);
		Group group = (Group) query.uniqueResult();
		
		// ajout du Site dans le Group
		group.getSiteAllowed().add(site);
		
		// update
		Transaction tx = null;
		try {
			tx = session.beginTransaction();
			session.update(group);
			tx.commit();
		}catch (Exception e) {
			e.printStackTrace();
			if (tx != null && session.isOpen())
				tx.rollback();
			return false;
		}finally{
			session.close();
		}
		
		return true;
	}
	
	public boolean addType(String groupName, String typeName){
		Session session = HibernateUtils.getSessionFactory().openSession();
		// récupération du Type
		Query query = session.createQuery("SELECT type FROM Type type WHERE type.name = :name");
		query.setParameter("name", typeName);
		Type type = (Type) query.uniqueResult();
		if (type == null){
			System.err.println("Group#addType() : nom du type inconnu !");
			session.close();
			return false;
		}
		
		// récupération du Group
		query = session.createQuery("SELECT group FROM Group group WHERE group.name = :name");
		query.setParameter("name", groupName);
		Group group = (Group) query.uniqueResult();
		
		// ajout du Type dans le Group
		group.getTypeAllowed().add(type);
		
		// update
		Transaction tx = null;
		try {
			tx = session.beginTransaction();
			session.update(group);
			tx.commit();
		}catch (Exception e) {
			e.printStackTrace();
			if (tx != null && session.isOpen())
				tx.rollback();
			return false;
		}finally{
			session.close();
		}
		
		return true;
	}

	/**
	 * Réinitialise tous les droits pour tous les groupes pour le mode DEMO.
	 */
	@SuppressWarnings("unchecked")
	public void resetForDemoMode(){
		Session session = HibernateUtils.getSessionFactory().openSession();
		Transaction tx = null;
		Query query = session.createQuery("SELECT group FROM Group group");
		List<Group> listGroup = query.list();
		for(Group g : listGroup){
			g.setCrudMessage(7);	// a le droit de tout faire avec les messages
			g.setCrudUser(7);		// n'a QUE le droit de voir les profils
			g.setCrudGroup(1);		// n'a QUE le droit de voir les groupes
			g.setCrudSite(1);		// n'a QUE le droit de voir les sites
			g.setCrudType(1);		// n'a QUE le droit de voir les types
			g.setCrudLog(1);		// n'a QUE le droit de voir les logs
			try {
				tx = session.beginTransaction();
				session.update(g);
				tx.commit();
			}catch (Exception e) {
				e.printStackTrace();
				if (tx != null && session.isOpen())
					tx.rollback();
			} 
		}
		
		
		
		session.close();
		
	}
}
