package fr.janua.server.entity;

import java.util.HashSet;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.Table;
import javax.persistence.Version;

@Entity(name = "Group")
@Table(name = "t_group")
public class Group {

	@Id
	@Column(name = "group_name", length = 40)
	private String name;

	@Column(name = "group_crud_user", length = 1)
	private int crudUser;

	@Column(name = "group_crud_message", length = 1)
	private int crudMessage;

	@Column(name = "group_crud_group", length = 1)
	private int crudGroup;

	@Column(name = "group_crud_type", length = 1)
	private int crudType;

	@Column(name = "group_crud_site", length = 1)
	private int crudSite;

	@Column(name = "group_crud_log", length = 1)
	private int crudLog;

	@Column(name = "group_crud_upload_doc", length = 1)
	private int crudUploadDoc;

	@Column(name = "group_mail")
	private String mail;

	@ManyToMany(cascade = CascadeType.ALL, fetch = FetchType.EAGER)
	@JoinTable(name = "ta_group_site", joinColumns = { @JoinColumn(name = "group_name") }, inverseJoinColumns = { @JoinColumn(name = "site_name") })
	private Set<Site> siteAllowed = new HashSet<Site>(0);

	@ManyToMany(cascade = CascadeType.ALL, fetch = FetchType.EAGER)
	@JoinTable(name = "ta_group_type", joinColumns = { @JoinColumn(name = "group_name") }, inverseJoinColumns = { @JoinColumn(name = "type_name") })
	private Set<Type> typeAllowed = new HashSet<Type>(0);

	@Version
	@Column(name = "site_version")
	// @NotNull
	private Integer version;

	/* Constructeurs */

	/**
	 * Constructeur vide.
	 */
	public Group() {
	}

	/**
	 * Constructeur par défaut de la classe Group.
	 * 
	 * @param name
	 * @param crudUser
	 * @param crudMessage
	 * @param crudGroup
	 * @param crudType
	 * @param crudSite
	 * @param crudLog
	 * @param crudUploadDoc
	 * @param mail
	 * @param siteAllowed
	 * @param typeAllowed
	 * @param version
	 */
	public Group(String name, int crudUser, int crudMessage, int crudGroup,
			int crudType, int crudSite, int crudLog, int crudUploadDoc,
			String mail, Set<Site> siteAllowed, Set<Type> typeAllowed,
			Integer version) {
		this.name = name;
		this.crudUser = crudUser;
		this.crudMessage = crudMessage;
		this.crudGroup = crudGroup;
		this.crudType = crudType;
		this.crudSite = crudSite;
		this.crudLog = crudLog;
		this.crudUploadDoc = crudUploadDoc;
		this.mail = mail;
		this.siteAllowed = siteAllowed;
		this.typeAllowed = typeAllowed;
		this.version = version;
	}

	/* Accesseurs / Mutateurs */

	/**
	 * @return the name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param name
	 *            the name to set
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @return the crudUser
	 */
	public int getCrudUser() {
		return crudUser;
	}

	/**
	 * @param crudUser
	 *            the crudUser to set
	 */
	public void setCrudUser(int crudUser) {
		this.crudUser = crudUser;
	}

	/**
	 * @return the crudMessage
	 */
	public int getCrudMessage() {
		return crudMessage;
	}

	/**
	 * @param crudMessage
	 *            the crudMessage to set
	 */
	public void setCrudMessage(int crudMessage) {
		this.crudMessage = crudMessage;
	}

	/**
	 * @return the crudGroup
	 */
	public int getCrudGroup() {
		return crudGroup;
	}

	/**
	 * @param crudGroup
	 *            the crudGroup to set
	 */
	public void setCrudGroup(int crudGroup) {
		this.crudGroup = crudGroup;
	}

	/**
	 * @return the crudType
	 */
	public int getCrudType() {
		return crudType;
	}

	/**
	 * @param crudType
	 *            the crudType to set
	 */
	public void setCrudType(int crudType) {
		this.crudType = crudType;
	}

	/**
	 * @return the crudSite
	 */
	public int getCrudSite() {
		return crudSite;
	}

	/**
	 * @param crudSite
	 *            the crudSite to set
	 */
	public void setCrudSite(int crudSite) {
		this.crudSite = crudSite;
	}

	/**
	 * @return the crudLog
	 */
	public int getCrudLog() {
		return crudLog;
	}

	/**
	 * @param crudLog
	 *            the crudLog to set
	 */
	public void setCrudLog(int crudLog) {
		this.crudLog = crudLog;
	}
	
	/**
	 * @return the crudUploadDoc
	 */
	public int getCrudUploadDoc() {
		return crudUploadDoc;
	}

	/**
	 * @param crudUploadDoc the crudUploadDoc to set
	 */
	public void setCrudUploadDoc(int crudUploadDoc) {
		this.crudUploadDoc = crudUploadDoc;
	}

	/**
	 * @return the mail
	 */
	public String getMail() {
		return mail;
	}

	/**
	 * @param mail
	 *            the mail to set
	 */
	public void setMail(String mail) {
		this.mail = mail;
	}

	/**
	 * @return the version
	 */
	public Integer getVersion() {
		return version;
	}

	/**
	 * @param version
	 *            the version to set
	 */
	public void setVersion(Integer version) {
		this.version = version;
	}

	/**
	 * @return the siteAllowed
	 */
	public Set<Site> getSiteAllowed() {
		return siteAllowed;
	}

	/**
	 * @param siteAllowed
	 *            the siteAllowed to set
	 */
	public void setSiteAllowed(Set<Site> siteAllowed) {
		this.siteAllowed = siteAllowed;
	}

	/**
	 * @return the typeAllowed
	 */
	public Set<Type> getTypeAllowed() {
		return typeAllowed;
	}

	/**
	 * @param typeAllowed
	 *            the typeAllowed to set
	 */
	public void setTypeAllowed(Set<Type> typeAllowed) {
		this.typeAllowed = typeAllowed;
	}

	@Override
	public String toString() {
		return "Group [name=" + name + ", crudUser=" + crudUser
				+ ", crudMessage=" + crudMessage + ", crudGroup=" + crudGroup
				+ ", crudType=" + crudType + ", crudSite=" + crudSite
				+ ", crudLog=" + crudLog + ", crudUploadDoc=" + crudUploadDoc
				+ ", mail=" + mail + ", siteAllowed=" + siteAllowed
				+ ", typeAllowed=" + typeAllowed + ", version=" + version + "]";
	}


	
}
