package fr.janua.server.entity;

import java.util.HashSet;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Version;

@Entity(name = "Message")
@Table(name = "t_message")
public class Message {
	@Id
	@Column(name = "message_id")
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	private int id;

	@Column(name = "message_title")
	private String title;

	@Column(name = "message_description", length=2500)
	private String description;

	@Column(name = "message_comment_closure", length=2500)
	private String commentClosure;

	@Column(name = "message_creation_date")
	private long creationDate;

	@Column(name = "message_closure_date")
	private long closureDate;

	@Column(name = "message_status")
	private int status;

	@Column(name = "message_event_date")
	private long eventDate;

	@Column(name = "message_last_change")
	private long lastChange;
	
//	@ManyToOne
//	@JoinColumn(name = "message_group_creator", referencedColumnName = "group_name")
	@Column(name = "message_group_creator")
	private String groupCreator;
	
	@Column(name = "message_group_closer")
	private String groupCloser;

	@Column(name = "message_name_creator")
	private String nameCreator;

	@Column(name = "message_name_closer")
	private String nameCloser;

	@ManyToOne
	@JoinColumn(name = "message_site", referencedColumnName = "site_name")
	private Site site;

	@ManyToMany(cascade = CascadeType.ALL, fetch = FetchType.EAGER)
	@JoinTable(name = "ta_message_types", joinColumns = { @JoinColumn(name = "message_id") }, inverseJoinColumns = { @JoinColumn(name = "type_name") })
	private Set<Type> types = new HashSet<Type>(0);

	// @OneToOne
	// @JoinColumn(name = "message_complement", referencedColumnName =
	// "complementary_message_id")

	// @OneToMany(mappedBy="comments")

	// Table d'association => choix de conception
	@OneToMany(cascade = { CascadeType.ALL }, fetch = FetchType.EAGER)
	@JoinTable(name = "ta_message_comments", joinColumns = { @JoinColumn(name = "message_id") }, inverseJoinColumns = { @JoinColumn(name = "comment_id") })
	private Set<Comment> comments = new HashSet<Comment>(0);

	@Version
	@Column(name = "message_version")
	// @NotNull
	private Integer version;
	/* Constructeurs */

	/**
	 * 
	 */
	public Message() {}


	public Message(int id, String title, String description,
			String commentClosure, long creationDate, long closureDate,
			int status, long eventDate, long lastChange, String groupCreator,
			String groupCloser, String nameCreator, String nameCloser,
			Site site, Set<Type> types, Set<Comment> comments, Integer version) {
		super();
		this.id = id;
		this.title = title;
		this.description = description;
		this.commentClosure = commentClosure;
		this.creationDate = creationDate;
		this.closureDate = closureDate;
		this.status = status;
		this.eventDate = eventDate;
		this.lastChange = lastChange;
		this.groupCreator = groupCreator;
		this.groupCloser = groupCloser;
		this.nameCreator = nameCreator;
		this.nameCloser = nameCloser;
		this.site = site;
		this.types = types;
		this.comments = comments;
		this.version = version;
	}





	/**
	 * @return the id
	 */
	public int getId() {
		return id;
	}

	/**
	 * @param id the id to set
	 */
	public void setId(int id) {
		this.id = id;
	}

	/**
	 * @return the title
	 */
	public String getTitle() {
		return title;
	}

	/**
	 * @param title the title to set
	 */
	public void setTitle(String title) {
		this.title = title;
	}

	/**
	 * @return the description
	 */
	public String getDescription() {
		return description;
	}

	/**
	 * @param description the description to set
	 */
	public void setDescription(String description) {
		this.description = description;
	}

	/**
	 * @return the commentClosure
	 */
	public String getCommentClosure() {
		return commentClosure;
	}

	/**
	 * @param commentClosure the commentClosure to set
	 */
	public void setCommentClosure(String commentClosure) {
		this.commentClosure = commentClosure;
	}

	/**
	 * @return the creationDate
	 */
	public long getCreationDate() {
		return creationDate;
	}

	/**
	 * @param creationDate the creationDate to set
	 */
	public void setCreationDate(long creationDate) {
		this.creationDate = creationDate;
	}

	/**
	 * @return the closureDate
	 */
	public long getClosureDate() {
		return closureDate;
	}

	/**
	 * @param closureDate the closureDate to set
	 */
	public void setClosureDate(long closureDate) {
		this.closureDate = closureDate;
	}

	/**
	 * @return the status
	 */
	public int getStatus() {
		return status;
	}

	/**
	 * @param status the status to set
	 */
	public void setStatus(int status) {
		this.status = status;
	}

	/**
	 * @return the eventDate
	 */
	public long getEventDate() {
		return eventDate;
	}

	/**
	 * @param eventDate the eventDate to set
	 */
	public void setEventDate(long eventDate) {
		this.eventDate = eventDate;
	}

	/**
	 * @return the groupCreator
	 */
	public String getGroupCreator() {
		return groupCreator;
	}

	/**
	 * @param groupCreator the groupCreator to set
	 */
	public void setGroupCreator(String groupCreator) {
		this.groupCreator = groupCreator;
	}

	/**
	 * @return the groupCloser
	 */
	public String getGroupCloser() {
		return groupCloser;
	}

	/**
	 * @param groupCloser the groupCloser to set
	 */
	public void setGroupCloser(String groupCloser) {
		this.groupCloser = groupCloser;
	}

	/**
	 * @return the nameCreator
	 */
	public String getNameCreator() {
		return nameCreator;
	}

	/**
	 * @param nameCreator the nameCreator to set
	 */
	public void setNameCreator(String nameCreator) {
		this.nameCreator = nameCreator;
	}

	/**
	 * @return the nameCloser
	 */
	public String getNameCloser() {
		return nameCloser;
	}

	/**
	 * @param nameCloser the nameCloser to set
	 */
	public void setNameCloser(String nameCloser) {
		this.nameCloser = nameCloser;
	}

	/**
	 * @return the site
	 */
	public Site getSite() {
		return site;
	}

	/**
	 * @param site the site to set
	 */
	public void setSite(Site site) {
		this.site = site;
	}

	/**
	 * @return the types
	 */
	public Set<Type> getTypes() {
		return types;
	}

	/**
	 * @param types the types to set
	 */
	public void setTypes(Set<Type> types) {
		this.types = types;
	}

	/**
	 * @return the comments
	 */
	public Set<Comment> getComments() {
		return comments;
	}

	/**
	 * @param comments the comments to set
	 */
	public void setComments(Set<Comment> comments) {
		this.comments = comments;
	}

	/**
	 * @return the version
	 */
	public Integer getVersion() {
		return version;
	}

	/**
	 * @param version the version to set
	 */
	public void setVersion(Integer version) {
		this.version = version;
	}


	/**
	 * @return the lastChange
	 */
	public long getLastChange() {
		return lastChange;
	}


	/**
	 * @param lastChange the lastChange to set
	 */
	public void setLastChange(long lastChange) {
		this.lastChange = lastChange;
	}


	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return "Message [id=" + id + ", title=" + title + ", description="
				+ description + ", commentClosure=" + commentClosure
				+ ", creationDate=" + creationDate + ", closureDate="
				+ closureDate + ", status=" + status + ", eventDate="
				+ eventDate + ", lastChange=" + lastChange + ", groupCreator="
				+ groupCreator + ", groupCloser=" + groupCloser
				+ ", nameCreator=" + nameCreator + ", nameCloser=" + nameCloser
				+ ", site=" + site + ", types=" + types + ", comments="
				+ comments + ", version=" + version + "]";
	}



	
	

	
	
	
	
}
