package fr.janua.server.entity;

import java.util.HashSet;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Version;


@Entity(name = "User")
@Table(name = "t_user")
public class User{
	
	@Id
	@Column(name="user_login", length=40)
	private String login;
	
	@Column(name="user_password", length=40)
	private String password;
	
	@Column(name="user_first_name", length=40)
	private String firstName;
	
	@Column(name="user_last_name", length=40)
	private String lastName;
	
	@Column(name="user_mail")
	private String mail;

	@Column(name="user_enabled")
	private boolean enabled;
	
	@Column(name="user_show_help")
	private boolean showHelp;
	
	@ManyToOne
	@JoinColumn(name="user_group", referencedColumnName="group_name")
	private Group group;
	
	@Column(name="user_last_activity")
	private long lastActivity;
	
	@ManyToMany(cascade = CascadeType.ALL, fetch = FetchType.EAGER)
	@JoinTable(name = "ta_user_site", joinColumns = { @JoinColumn(name = "user_name") }, inverseJoinColumns = { @JoinColumn(name = "site_name") })
	private Set<Site> siteAllowed = new HashSet<Site>(0);
	
	@Version
	@Column(name = "user_version")
//	@NotNull
	private Integer version;
	
	
	
	/* Constructeurs */
	
	/**
	 * Constructeur vide.
	 */
	public User() {}



	/**
	 * @param login
	 * @param password
	 * @param firstName
	 * @param lastName
	 * @param mail
	 * @param enabled
	 * @param showHelp
	 * @param group
	 * @param lastActivity
	 * @param siteAllowed
	 * @param version
	 */
	public User(String login, String password, String firstName,
			String lastName, String mail, boolean enabled, boolean showHelp,
			Group group, long lastActivity, Set<Site> siteAllowed,
			Integer version) {
		this.login = login;
		this.password = password;
		this.firstName = firstName;
		this.lastName = lastName;
		this.mail = mail;
		this.enabled = enabled;
		this.showHelp = showHelp;
		this.group = group;
		this.lastActivity = lastActivity;
		this.siteAllowed = siteAllowed;
		this.version = version;
	}



	/**
	 * @return the login
	 */
	public String getLogin() {
		return login;
	}



	/**
	 * @param login the login to set
	 */
	public void setLogin(String login) {
		this.login = login;
	}



	/**
	 * @return the password
	 */
	public String getPassword() {
		return password;
	}



	/**
	 * @param password the password to set
	 */
	public void setPassword(String password) {
		this.password = password;
	}



	/**
	 * @return the firstName
	 */
	public String getFirstName() {
		return firstName;
	}



	/**
	 * @param firstName the firstName to set
	 */
	public void setFirstName(String firstName) {
		this.firstName = firstName;
	}



	/**
	 * @return the lastName
	 */
	public String getLastName() {
		return lastName;
	}



	/**
	 * @param lastName the lastName to set
	 */
	public void setLastName(String lastName) {
		this.lastName = lastName;
	}



	/**
	 * @return the mail
	 */
	public String getMail() {
		return mail;
	}



	/**
	 * @param mail the mail to set
	 */
	public void setMail(String mail) {
		this.mail = mail;
	}



	/**
	 * @return the enabled
	 */
	public boolean isEnabled() {
		return enabled;
	}



	/**
	 * @param enabled the enabled to set
	 */
	public void setEnabled(boolean enabled) {
		this.enabled = enabled;
	}



	/**
	 * @return the showHelp
	 */
	public boolean isShowHelp() {
		return showHelp;
	}



	/**
	 * @param showHelp the showHelp to set
	 */
	public void setShowHelp(boolean showHelp) {
		this.showHelp = showHelp;
	}



	/**
	 * @return the group
	 */
	public Group getGroup() {
		return group;
	}



	/**
	 * @param group the group to set
	 */
	public void setGroup(Group group) {
		this.group = group;
	}



	/**
	 * @return the lastActivity
	 */
	public long getLastActivity() {
		return lastActivity;
	}



	/**
	 * @param lastActivity the lastActivity to set
	 */
	public void setLastActivity(long lastActivity) {
		this.lastActivity = lastActivity;
	}



	/**
	 * @return the siteAllowed
	 */
	public Set<Site> getSiteAllowed() {
		return siteAllowed;
	}



	/**
	 * @param siteAllowed the siteAllowed to set
	 */
	public void setSiteAllowed(Set<Site> siteAllowed) {
		this.siteAllowed = siteAllowed;
	}



	/**
	 * @return the version
	 */
	public Integer getVersion() {
		return version;
	}



	/**
	 * @param version the version to set
	 */
	public void setVersion(Integer version) {
		this.version = version;
	}



	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return "User [login=" + login + ", password=" + password
				+ ", firstName=" + firstName + ", lastName=" + lastName
				+ ", mail=" + mail + ", enabled=" + enabled + ", showHelp="
				+ showHelp + ", group=" + group + ", lastActivity="
				+ lastActivity + ", siteAllowed=" + siteAllowed + ", version="
				+ version + "]";
	}



	
}
