package fr.janua.server.utils;

import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.AnnotationConfiguration;

/**
 * Gestion des Sessions Hibernate.
 * @author gilles
 *
 */
public class HibernateUtils {
	private static SessionFactory sessionFactory;
	private static AnnotationConfiguration conf;

	/**
	 *  Crée une unique instance de la SessionFactory à partir de hibernate.cfg.xml
	 */
	static {
		try {
			conf = new AnnotationConfiguration().configure();
			sessionFactory = conf.buildSessionFactory();
		} catch (HibernateException ex) {
			throw new RuntimeException("Problème de configuration : " + ex.getMessage(), ex);
		}
	}

	/**
	 *  Renvoie une session Hibernate
	 * @return
	 * @throws HibernateException
	 */
	public static Session getSession() throws HibernateException {
		return sessionFactory.openSession();
	}

	public static SessionFactory getSessionFactory() {
		return sessionFactory;
	}

	/**
	 * Change les données pour se connecter à la base de donnée.
	 * @param hostname
	 * @param database
	 * @param username
	 * @param password
	 * @return
	 */
	public static boolean changeConfig(String hostname, String database, String username, String password){
		conf.setProperty("hibernate.connection.url", "jdbc:mysql://" + hostname + "/" + database);
		conf.setProperty("hibernate.connection.username", username);
		conf.setProperty("hibernate.connection.password", password);
		sessionFactory = conf.buildSessionFactory();
		return checkConnection();
	}
	
	/**
	 * Retourne <code>true</code> si la connexion avec la base de donnée est correctement configurée.
	 * @return
	 */
	public static boolean checkConnection(){
		Session s = sessionFactory.openSession();
		try{
			Query query = s.createQuery("FROM User");
			query.setMaxResults(1);
			query.uniqueResult();
			return true;
		}catch(Exception e){
			e.printStackTrace();
			System.out.println("open session > fail !!");
			return false;
		}finally{
			s.close();
		}		
	}
	
	public static String getUrl(){
		return conf.getProperty("hibernate.connection.url");
	}
	
	public static String getUsername(){
		return conf.getProperty("hibernate.connection.username");
	}
	
	/**
	 * Recharge les configuration de la base de données.
	 */
	public static void reloadConf(){
		conf = new AnnotationConfiguration().configure();
		sessionFactory = conf.buildSessionFactory();
	}
}